@@ -6,131 +6,61 @@
  * expressions that go into TargetGroup classes.                           *
  *                                                                         *
  ***********************IMPORTANT NMAP LICENSE TERMS************************
- *                                                                         *
- * The Nmap Security Scanner is (C) 1996-2017 Insecure.Com LLC ("The Nmap  *
- * Project"). Nmap is also a registered trademark of the Nmap Project.     *
- * This program is free software; you may redistribute and/or modify it    *
- * under the terms of the GNU General Public License as published by the   *
- * Free Software Foundation; Version 2 ("GPL"), BUT ONLY WITH ALL OF THE   *
- * CLARIFICATIONS AND EXCEPTIONS DESCRIBED HEREIN.  This guarantees your   *
- * right to use, modify, and redistribute this software under certain      *
- * conditions.  If you wish to embed Nmap technology into proprietary      *
- * software, we sell alternative licenses (contact sales@nmap.com).        *
- * Dozens of software vendors already license Nmap technology such as      *
- * host discovery, port scanning, OS detection, version detection, and     *
- * the Nmap Scripting Engine.                                              *
- *                                                                         *
- * Note that the GPL places important restrictions on "derivative works",  *
- * yet it does not provide a detailed definition of that term.  To avoid   *
- * misunderstandings, we interpret that term as broadly as copyright law   *
- * allows.  For example, we consider an application to constitute a        *
- * derivative work for the purpose of this license if it does any of the   *
- * following with any software or content covered by this license          *
- * ("Covered Software"):                                                   *
- *                                                                         *
- * o Integrates source code from Covered Software.                         *
- *                                                                         *
- * o Reads or includes copyrighted data files, such as Nmap's nmap-os-db   *
- * or nmap-service-probes.                                                 *
- *                                                                         *
- * o Is designed specifically to execute Covered Software and parse the    *
- * results (as opposed to typical shell or execution-menu apps, which will *
- * execute anything you tell them to).                                     *
- *                                                                         *
- * o Includes Covered Software in a proprietary executable installer.  The *
- * installers produced by InstallShield are an example of this.  Including *
- * Nmap with other software in compressed or archival form does not        *
- * trigger this provision, provided appropriate open source decompression  *
- * or de-archiving software is widely available for no charge.  For the    *
- * purposes of this license, an installer is considered to include Covered *
- * Software even if it actually retrieves a copy of Covered Software from  *
- * another source during runtime (such as by downloading it from the       *
- * Internet).                                                              *
- *                                                                         *
- * o Links (statically or dynamically) to a library which does any of the  *
- * above.                                                                  *
- *                                                                         *
- * o Executes a helper program, module, or script to do any of the above.  *
- *                                                                         *
- * This list is not exclusive, but is meant to clarify our interpretation  *
- * of derived works with some common examples.  Other people may interpret *
- * the plain GPL differently, so we consider this a special exception to   *
- * the GPL that we apply to Covered Software.  Works which meet any of     *
- * these conditions must conform to all of the terms of this license,      *
- * particularly including the GPL Section 3 requirements of providing      *
- * source code and allowing free redistribution of the work as a whole.    *
- *                                                                         *
- * As another special exception to the GPL terms, the Nmap Project grants  *
- * permission to link the code of this program with any version of the     *
- * OpenSSL library which is distributed under a license identical to that  *
- * listed in the included docs/licenses/OpenSSL.txt file, and distribute   *
- * linked combinations including the two.                                  *
- *                                                                         *
- * The Nmap Project has permission to redistribute Npcap, a packet         *
- * capturing driver and library for the Microsoft Windows platform.        *
- * Npcap is a separate work with it's own license rather than this Nmap    *
- * license.  Since the Npcap license does not permit redistribution        *
- * without special permission, our Nmap Windows binary packages which      *
- * contain Npcap may not be redistributed without special permission.      *
- *                                                                         *
- * Any redistribution of Covered Software, including any derived works,    *
- * must obey and carry forward all of the terms of this license, including *
- * obeying all GPL rules and restrictions.  For example, source code of    *
- * the whole work must be provided and free redistribution must be         *
- * allowed.  All GPL references to "this License", are to be treated as    *
- * including the terms and conditions of this license text as well.        *
- *                                                                         *
- * Because this license imposes special exceptions to the GPL, Covered     *
- * Work may not be combined (even as part of a larger work) with plain GPL *
- * software.  The terms, conditions, and exceptions of this license must   *
- * be included as well.  This license is incompatible with some other open *
- * source licenses as well.  In some cases we can relicense portions of    *
- * Nmap or grant special permissions to use it in other open source        *
- * software.  Please contact fyodor@nmap.org with any such requests.       *
- * Similarly, we don't incorporate incompatible open source software into  *
- * Covered Software without special permission from the copyright holders. *
- *                                                                         *
- * If you have any questions about the licensing restrictions on using     *
- * Nmap in other works, are happy to help.  As mentioned above, we also    *
- * offer alternative license to integrate Nmap into proprietary            *
- * applications and appliances.  These contracts have been sold to dozens  *
- * of software vendors, and generally include a perpetual license as well  *
- * as providing for priority support and updates.  They also fund the      *
- * continued development of Nmap.  Please email sales@nmap.com for further *
- * information.                                                            *
- *                                                                         *
- * If you have received a written license agreement or contract for        *
- * Covered Software stating terms other than these, you may choose to use  *
- * and redistribute Covered Software under those terms instead of these.   *
- *                                                                         *
- * Source is provided to this software because we believe users have a     *
- * right to know exactly what a program is going to do before they run it. *
- * This also allows you to audit the software for security holes.          *
- *                                                                         *
- * Source code also allows you to port Nmap to new platforms, fix bugs,    *
- * and add new features.  You are highly encouraged to send your changes   *
- * to the dev@nmap.org mailing list for possible incorporation into the    *
- * main distribution.  By sending these changes to Fyodor or one of the    *
- * Insecure.Org development mailing lists, or checking them into the Nmap  *
- * source code repository, it is understood (unless you specify            *
- * otherwise) that you are offering the Nmap Project the unlimited,        *
- * non-exclusive right to reuse, modify, and relicense the code.  Nmap     *
- * will always be available Open Source, but this is important because     *
- * the inability to relicense code has caused devastating problems for     *
- * other Free Software projects (such as KDE and NASM).  We also           *
- * occasionally relicense the code to third parties as discussed above.    *
- * If you wish to specify special license conditions of your               *
- * contributions, just say so when you send them.                          *
- *                                                                         *
- * This program is distributed in the hope that it will be useful, but     *
- * WITHOUT ANY WARRANTY; without even the implied warranty of              *
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the Nmap      *
- * license file for more details (it's in a COPYING file included with     *
- * Nmap, and also available from https://svn.nmap.org/nmap/COPYING)        *
- *                                                                         *
+ *
+ * The Nmap Security Scanner is (C) 1996-2023 Nmap Software LLC ("The Nmap
+ * Project"). Nmap is also a registered trademark of the Nmap Project.
+ *
+ * This program is distributed under the terms of the Nmap Public Source
+ * License (NPSL). The exact license text applying to a particular Nmap
+ * release or source code control revision is contained in the LICENSE
+ * file distributed with that version of Nmap or source code control
+ * revision. More Nmap copyright/legal information is available from
+ * https://nmap.org/book/man-legal.html, and further information on the
+ * NPSL license itself can be found at https://nmap.org/npsl/ . This
+ * header summarizes some key points from the Nmap license, but is no
+ * substitute for the actual license text.
+ *
+ * Nmap is generally free for end users to download and use themselves,
+ * including commercial use. It is available from https://nmap.org.
+ *
+ * The Nmap license generally prohibits companies from using and
+ * redistributing Nmap in commercial products, but we sell a special Nmap
+ * OEM Edition with a more permissive license and special features for
+ * this purpose. See https://nmap.org/oem/
+ *
+ * If you have received a written Nmap license agreement or contract
+ * stating terms other than these (such as an Nmap OEM license), you may
+ * choose to use and redistribute Nmap under those terms instead.
+ *
+ * The official Nmap Windows builds include the Npcap software
+ * (https://npcap.com) for packet capture and transmission. It is under
+ * separate license terms which forbid redistribution without special
+ * permission. So the official Nmap Windows builds may not be redistributed
+ * without special permission (such as an Nmap OEM license).
+ *
+ * Source is provided to this software because we believe users have a
+ * right to know exactly what a program is going to do before they run it.
+ * This also allows you to audit the software for security holes.
+ *
+ * Source code also allows you to port Nmap to new platforms, fix bugs, and add
+ * new features. You are highly encouraged to submit your changes as a Github PR
+ * or by email to the dev@nmap.org mailing list for possible incorporation into
+ * the main distribution. Unless you specify otherwise, it is understood that
+ * you are offering us very broad rights to use your submissions as described in
+ * the Nmap Public Source License Contributor Agreement. This is important
+ * because we fund the project by selling licenses with various terms, and also
+ * because the inability to relicense code has caused devastating problems for
+ * other Free Software projects (such as KDE and NASM).
+ *
+ * The free version of Nmap is distributed in the hope that it will be
+ * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. Warranties,
+ * indemnification and commercial support are all available through the
+ * Npcap OEM program--see https://nmap.org/oem/
+ *
  ***************************************************************************/
 
-/* $Id: TargetGroup.cc 36788 2017-06-07 12:32:38Z dmiller $ */
+/* $Id: TargetGroup.cc 38653 2023-04-14 17:11:46Z dmiller $ */
 
 #include "tcpip.h"
 #include "TargetGroup.h"
@@ -139,8 +69,17 @@
 #include "nmap.h"
 #include "libnetutil/netutil.h"
 
+#include <string>
 #include <sstream>
 #include <errno.h>
+#include <limits.h> // CHAR_BIT
+
+/* We use bit vectors to represent what values are allowed in an IPv4 octet.
+   Each vector is built up of an array of bitvector_t (any convenient integer
+   type). */
+typedef unsigned long bitvector_t;
+/* A 256-element bit vector, representing legal values for one octet. */
+typedef bitvector_t octet_bitvector[(256 - 1) / (sizeof(unsigned long) * CHAR_BIT) + 1];
 
 #define BITVECTOR_BITS (sizeof(bitvector_t) * CHAR_BIT)
 #define BIT_SET(v, n) ((v)[(n) / BITVECTOR_BITS] |= 1UL << ((n) % BITVECTOR_BITS))
@@ -148,7 +87,77 @@
 
 extern NmapOps o;
 
-NewTargets *NewTargets::new_targets;
+class NetBlock {
+public:
+  virtual ~NetBlock() {}
+  NetBlock() {
+    current_addr = resolvedaddrs.begin();
+    }
+  std::string hostname;
+  std::list<struct sockaddr_storage> resolvedaddrs;
+  std::list<struct sockaddr_storage> unscanned_addrs;
+  std::list<struct sockaddr_storage>::const_iterator current_addr;
+
+  /* Parses an expression such as 192.168.0.0/16, 10.1.0-5.1-254, or
+     fe80::202:e3ff:fe14:1102/112 and returns a newly allocated NetBlock. The af
+     parameter is AF_INET or AF_INET6. Returns NULL in case of error. */
+  static NetBlock *parse_expr(const char *target_expr, int af);
+
+  bool is_resolved_address(const struct sockaddr_storage *ss) const;
+
+  /* For NetBlock subclasses that need to "resolve" themselves into a different
+   * NetBlock subclass, override this method. Otherwise, it's safe to reassign
+   * the return value to the pointer that this method was called through.
+   * On error, return NULL. */
+  virtual NetBlock *resolve() { return this; }
+  virtual bool next(struct sockaddr_storage *ss, size_t *sslen) = 0;
+  virtual void apply_netmask(int bits) = 0;
+  virtual std::string str() const = 0;
+};
+
+class NetBlockIPv4Ranges : public NetBlock {
+public:
+  octet_bitvector octets[4];
+
+  NetBlockIPv4Ranges();
+
+  bool next(struct sockaddr_storage *ss, size_t *sslen);
+  void apply_netmask(int bits);
+  std::string str() const;
+  void set_addr(const struct sockaddr_in *addr);
+
+private:
+  unsigned int counter[4];
+};
+
+class NetBlockIPv6Netmask : public NetBlock {
+public:
+  void set_addr(const struct sockaddr_in6 *addr);
+
+  bool next(struct sockaddr_storage *ss, size_t *sslen);
+  void apply_netmask(int bits);
+  std::string str() const;
+
+private:
+  bool exhausted;
+  struct sockaddr_in6 addr;
+  struct in6_addr start;
+  struct in6_addr cur;
+  struct in6_addr end;
+};
+
+class NetBlockHostname : public NetBlock {
+public:
+  NetBlockHostname(const char *hostname, int af);
+  int af;
+  int bits;
+
+  NetBlock *resolve();
+
+  bool next(struct sockaddr_storage *ss, size_t *sslen);
+  void apply_netmask(int bits);
+  std::string str() const;
+};
 
 /* Return a newly allocated string containing the part of expr up to the last
    '/' (or a copy of the whole string if there is no slash). *bits will contain
@@ -160,7 +169,7 @@ static char *split_netmask(const char *expr, int *bits) {
   slash = strrchr(expr, '/');
   if (slash != NULL) {
     long l;
-    char *tail;
+    const char *tail;
 
     l = parse_long(slash + 1, &tail);
     if (tail == slash + 1 || *tail != '\0' || l < 0 || l > INT_MAX)
@@ -192,7 +201,7 @@ static int parse_ipv4_ranges(octet_bitvector octets[4], const char *spec) {
     } else {
       for (;;) {
         long start, end;
-        char *tail;
+        const char *tail;
 
         errno = 0;
         start = parse_long(p, &tail);
@@ -260,6 +269,10 @@ static NetBlock *parse_expr_without_netmask(const char *hostexp, int af) {
 
   sslen = sizeof(ss);
   if (resolve_numeric(hostexp, 0, &ss, &sslen, AF_INET6) == 0) {
+    if (af != AF_INET6) {
+      error("%s looks like an IPv6 target specification -- you have to use the -6 option.", hostexp);
+      return NULL;
+    }
     NetBlockIPv6Netmask *netblock_ipv6;
 
     netblock_ipv6 = new NetBlockIPv6Netmask();
@@ -302,27 +315,20 @@ bail:
   return NULL;
 }
 
-/* Returns the first address which matches the address family af */
-static const struct sockaddr_storage *first_af_address(const std::list<struct sockaddr_storage> *addrs, int af) {
-  for (std::list<struct sockaddr_storage>::const_iterator it = addrs->begin(), end = addrs->end(); it != end; ++it) {
-    if (it->ss_family == af) {
-      return &*it;
+bool NetBlock::is_resolved_address(const struct sockaddr_storage *ss) const {
+  for (std::list<struct sockaddr_storage>::const_iterator it = this->resolvedaddrs.begin(), end = this->resolvedaddrs.end(); it != end; ++it) {
+    if (sockaddr_storage_equal(&*it, ss)) {
+      return true;
     }
   }
-  return NULL;
-}
-
-bool NetBlock::is_resolved_address(const struct sockaddr_storage *ss) const {
-  if (this->resolvedaddrs.empty())
-    return false;
-  return sockaddr_storage_equal(first_af_address(&this->resolvedaddrs, ss->ss_family), ss);
+  return false;
 }
 
 NetBlockIPv4Ranges::NetBlockIPv4Ranges() {
   unsigned int i;
 
+  memset(this->octets, 0, sizeof(this->octets));
   for (i = 0; i < 4; i++) {
-    memset(this->octets, 0, sizeof(this->octets));
     this->counter[i] = 0;
   }
 }
@@ -367,11 +373,16 @@ bool NetBlockIPv4Ranges::next(struct sockaddr_storage *ss, size_t *sslen) {
       break;
   }
   if (i >= 4) {
-    /* We cycled all counters. Mark them invalid for the next call. */
-    this->counter[0] = 256;
-    this->counter[1] = 256;
-    this->counter[2] = 256;
-    this->counter[3] = 256;
+    if (o.resolve_all && !this->resolvedaddrs.empty() && current_addr != this->resolvedaddrs.end() && ++current_addr != this->resolvedaddrs.end()) {
+      this->set_addr((struct sockaddr_in *) &*current_addr);
+    }
+    else {
+      /* We cycled all counters. Mark them invalid for the next call. */
+      this->counter[0] = 256;
+      this->counter[1] = 256;
+      this->counter[2] = 256;
+      this->counter[3] = 256;
+    }
   }
 
   return true;
@@ -475,7 +486,24 @@ std::string NetBlockIPv4Ranges::str() const {
   return result.str();
 }
 
+void NetBlockIPv4Ranges::set_addr(const struct sockaddr_in *addr) {
+  uint32_t ip;
+
+  assert(addr->sin_family == AF_INET);
+  ip = ntohl(addr->sin_addr.s_addr);
+  memset(this->octets, 0, sizeof(this->octets));
+  BIT_SET(this->octets[0], (ip & 0xFF000000) >> 24);
+  BIT_SET(this->octets[1], (ip & 0x00FF0000) >> 16);
+  BIT_SET(this->octets[2], (ip & 0x0000FF00) >> 8);
+  BIT_SET(this->octets[3], (ip & 0x000000FF));
+  /* Reset counter so that set_addr can be used to reset the whole NetBlock */
+  for (int i = 0; i < 4; i++) {
+    this->counter[i] = 0;
+  }
+}
+
 void NetBlockIPv6Netmask::set_addr(const struct sockaddr_in6 *addr) {
+  assert(addr->sin6_family == AF_INET6);
   this->exhausted = false;
   this->addr = *addr;
   this->start = this->addr.sin6_addr;
@@ -505,8 +533,14 @@ static bool ipv6_equal(const struct in6_addr *a, const struct in6_addr *b) {
 bool NetBlockIPv6Netmask::next(struct sockaddr_storage *ss, size_t *sslen) {
   struct sockaddr_in6 *sin6;
 
-  if (this->exhausted)
-    return false;
+  if (this->exhausted){
+    if (o.resolve_all && !this->resolvedaddrs.empty() && current_addr != this->resolvedaddrs.end() && ++current_addr != this->resolvedaddrs.end()) {
+      this->set_addr((struct sockaddr_in6 *) &*current_addr);
+    }
+    else {
+      return false;
+    }
+  }
 
   memset(ss, 0, sizeof(*ss));
   sin6 = (struct sockaddr_in6 *) ss;
@@ -624,11 +658,11 @@ std::string NetBlockIPv6Netmask::str() const {
   return result.str();
 }
 
-NetBlock *NetBlockHostname::resolve() const {
+NetBlock *NetBlockHostname::resolve() {
   struct addrinfo *addrs, *addr;
   std::list<struct sockaddr_storage> resolvedaddrs;
+  std::list<struct sockaddr_storage> unscanned_addrs;
   NetBlock *netblock;
-  const struct sockaddr_storage *sp = NULL;
   struct sockaddr_storage ss;
   size_t sslen;
 
@@ -636,17 +670,21 @@ NetBlock *NetBlockHostname::resolve() const {
   for (addr = addrs; addr != NULL; addr = addr->ai_next) {
     if (addr->ai_addrlen < sizeof(ss)) {
       memcpy(&ss, addr->ai_addr, addr->ai_addrlen);
-      resolvedaddrs.push_back(ss);
+      if ((o.resolve_all || resolvedaddrs.empty()) && addr->ai_family == this->af) {
+        resolvedaddrs.push_back(ss);
+      }
+      else {
+        unscanned_addrs.push_back(ss);
+      }
     }
   }
   if (addrs != NULL)
     freeaddrinfo(addrs);
 
-  if (resolvedaddrs.empty())
-    return NULL;
+  if (resolvedaddrs.empty()) {
+    if (unscanned_addrs.empty())
+      return NULL;
 
-  sp = first_af_address(&resolvedaddrs, this->af);
-  if (sp == NULL || sp->ss_family != this->af) {
     switch (this->af) {
       case AF_INET:
         error("Warning: Hostname %s resolves, but not to any IPv4 address. Try scanning with -6", this->hostname.c_str());
@@ -660,25 +698,20 @@ NetBlock *NetBlockHostname::resolve() const {
     }
     return NULL;
   }
-  ss = *sp;
+  ss = resolvedaddrs.front();
   sslen = sizeof(ss);
 
-  if (resolvedaddrs.size() > 1 && o.verbose > 1) {
+  if (!unscanned_addrs.empty() && o.verbose > 1) {
     error("Warning: Hostname %s resolves to %lu IPs. Using %s.", this->hostname.c_str(),
-      (unsigned long) resolvedaddrs.size(), inet_ntop_ez(&ss, sslen));
+      (unsigned long) unscanned_addrs.size() + resolvedaddrs.size(), inet_ntop_ez(&ss, sslen));
   }
 
   netblock = NULL;
   if (ss.ss_family == AF_INET) {
     NetBlockIPv4Ranges *netblock_ranges;
-    uint32_t ip;
 
-    ip = ntohl(((struct sockaddr_in *) &ss)->sin_addr.s_addr);
     netblock_ranges = new NetBlockIPv4Ranges();
-    BIT_SET(netblock_ranges->octets[0], (ip & 0xFF000000) >> 24);
-    BIT_SET(netblock_ranges->octets[1], (ip & 0x00FF0000) >> 16);
-    BIT_SET(netblock_ranges->octets[2], (ip & 0x0000FF00) >> 8);
-    BIT_SET(netblock_ranges->octets[3], (ip & 0x000000FF));
+    netblock_ranges->set_addr((struct sockaddr_in *) &ss);
     netblock = netblock_ranges;
   } else if (ss.ss_family == AF_INET6) {
     NetBlockIPv6Netmask *netblock_ipv6;
@@ -693,6 +726,8 @@ NetBlock *NetBlockHostname::resolve() const {
 
   netblock->hostname = this->hostname;
   netblock->resolvedaddrs = resolvedaddrs;
+  netblock->unscanned_addrs = unscanned_addrs;
+  netblock->current_addr = netblock->resolvedaddrs.begin();
   netblock->apply_netmask(this->bits);
 
   return netblock;
@@ -723,103 +758,81 @@ std::string NetBlockHostname::str() const {
   return result.str();
 }
 
-/* debug level for the adding target is: 3 */
-NewTargets *NewTargets::get (void) {
-  if (new_targets)
-    return new_targets;
-  new_targets = new NewTargets();
-  return new_targets;
+TargetGroup::~TargetGroup() {
+  if (this->netblock != NULL)
+    delete this->netblock;
 }
 
-NewTargets::NewTargets (void) {
-  Initialize();
+/* Initializes (or reinitializes) the object with a new expression, such
+   as 192.168.0.0/16 , 10.1.0-5.1-254 , or fe80::202:e3ff:fe14:1102 .
+   Returns 0 for success */
+int TargetGroup::parse_expr(const char *target_expr, int af) {
+  if (this->netblock != NULL)
+    delete this->netblock;
+  this->netblock = NetBlock::parse_expr(target_expr, af);
+  if (this->netblock != NULL)
+    return 0;
+  else
+    return 1;
 }
 
-void NewTargets::Initialize (void) {
-  history.clear();
-  while (!queue.empty())
-    queue.pop();
-}
+/* Grab the next host from this expression (if any) and updates its internal
+   state to reflect that the IP was given out.  Returns 0 and
+   fills in ss if successful.  ss must point to a pre-allocated
+   sockaddr_storage structure */
+int TargetGroup::get_next_host(struct sockaddr_storage *ss, size_t *sslen) {
+  if (this->netblock == NULL)
+    return -1;
 
-/* This private method is used to push new targets to the
- * queue. It returns the number of targets in the queue. */
-unsigned long NewTargets::push (const char *target) {
-  std::pair<std::set<std::string>::iterator, bool> pair_iter;
-  std::string tg(target);
-
-  if (tg.length() > 0) {
-    /* save targets in the scanned history here (NSE side). */
-    pair_iter = history.insert(tg);
-
-    /* A new target */
-    if (pair_iter.second == true) {
-      /* push target onto the queue for future scans */
-      queue.push(tg);
-
-      if (o.debugging > 2)
-        log_write(LOG_PLAIN, "New Targets: target %s pushed onto the queue.\n", tg.c_str());
-    } else {
-      if (o.debugging > 2)
-        log_write(LOG_PLAIN, "New Targets: target %s is already in the queue.\n", tg.c_str());
-      /* Return 1 when the target is already in the history cache,
-       * this will prevent returning 0 when the target queue is
-       * empty since no target was added. */
-      return 1;
+  /* If all we have at this point is a hostname and netmask, resolve into
+     something where we know the address. If we ever have to use strictly the
+     hostname, without doing local DNS resolution (like with a proxy scan), this
+     has to be made conditional (and perhaps an error if the netmask doesn't
+     limit it to exactly one address). */
+  NetBlock *netblock_resolved = this->netblock->resolve();
+  if (netblock_resolved != NULL) {
+    /* resolve may return the original netblock if it's not a type that needs
+     * to be resolved. Don't delete it! */
+    if (netblock_resolved != this->netblock) {
+      delete this->netblock;
+      this->netblock = netblock_resolved;
     }
   }
-
-  return queue.size();
-}
-
-/* Reads a target from the queue and return it to be pushed
- * onto Nmap scan queue */
-std::string NewTargets::read (void) {
-  std::string str;
-
-  /* check to see it there are targets in the queue */
-  if (!new_targets->queue.empty()) {
-    str = new_targets->queue.front();
-    new_targets->queue.pop();
+  else {
+    error("Failed to resolve \"%s\".", this->netblock->hostname.c_str());
+    if (this->netblock->hostname == "-")
+      error("Bare '-': did you put a space between '--'?");
+    return -1;
   }
 
-  return str;
+  if (this->netblock->next(ss, sslen))
+    return 0;
+  else
+    return -1;
 }
 
-void NewTargets::clear (void) {
-  new_targets->history.clear();
+/* Returns true iff the given address is the one that was resolved to create
+   this target group; i.e., not one of the addresses derived from it with a
+   netmask. */
+bool TargetGroup::is_resolved_address(const struct sockaddr_storage *ss) const {
+  return this->netblock->is_resolved_address(ss);
 }
 
-unsigned long NewTargets::get_number (void) {
-  return new_targets->history.size();
+/* Return a string of the name or address that was resolved for this group. */
+const char *TargetGroup::get_resolved_name(void) const {
+  if (this->netblock->hostname.empty())
+    return NULL;
+  else
+    return this->netblock->hostname.c_str();
 }
 
-unsigned long NewTargets::get_scanned (void) {
-  return new_targets->history.size() - new_targets->queue.size();
+/* Return the list of addresses that the name for this group resolved to, but
+   which were not scanned, if it came from a name resolution. */
+const std::list<struct sockaddr_storage> &TargetGroup::get_unscanned_addrs(void) const {
+  return this->netblock->unscanned_addrs;
 }
 
-unsigned long NewTargets::get_queued (void) {
-  return new_targets->queue.size();
-}
-
-/* This is the function that is used by nse_nmaplib.cc to add
- * new targets.
- * Returns the number of targets in the queue on success, or 0 on
- * failures or when the queue is empty. */
-unsigned long NewTargets::insert (const char *target) {
-  if (*target) {
-    if (new_targets == NULL) {
-      error("ERROR: to add targets run with -sC or --script options.");
-      return 0;
-    }
-    if (o.current_scantype == SCRIPT_POST_SCAN) {
-      error("ERROR: adding targets is disabled in the Post-scanning phase.");
-      return 0;
-    }
-    if (strlen(target) >= 1024) {
-      error("ERROR: new target is too long (>= 1024), failed to add it.");
-      return 0;
-    }
-  }
-
-  return new_targets->push(target);
+/* is the current expression a named host */
+int TargetGroup::get_namedhost() const {
+  return this->get_resolved_name() != NULL;
 }
