@@ -8,63 +8,61 @@
  * use in the Nmap Security Scanner ( https://nmap.org ).                   *
  *                                                                         *
  ***********************IMPORTANT NMAP LICENSE TERMS************************
- *                                                                         *
- * The Nmap Security Scanner is (C) 1996-2020 Insecure.Com LLC ("The Nmap  *
- * Project"). Nmap is also a registered trademark of the Nmap Project.     *
- *                                                                         *
- * This program is distributed under the terms of the Nmap Public Source   *
- * License (NPSL). The exact license text applying to a particular Nmap    *
- * release or source code control revision is contained in the LICENSE     *
- * file distributed with that version of Nmap or source code control       *
- * revision. More Nmap copyright/legal information is available from       *
- * https://nmap.org/book/man-legal.html, and further information on the    *
- * NPSL license itself can be found at https://nmap.org/npsl. This header  *
- * summarizes some key points from the Nmap license, but is no substitute  *
- * for the actual license text.                                            *
- *                                                                         *
- * Nmap is generally free for end users to download and use themselves,    *
- * including commercial use. It is available from https://nmap.org.        *
- *                                                                         *
- * The Nmap license generally prohibits companies from using and           *
- * redistributing Nmap in commercial products, but we sell a special Nmap  *
- * OEM Edition with a more permissive license and special features for     *
- * this purpose. See https://nmap.org/oem                                  *
- *                                                                         *
- * If you have received a written Nmap license agreement or contract       *
- * stating terms other than these (such as an Nmap OEM license), you may   *
- * choose to use and redistribute Nmap under those terms instead.          *
- *                                                                         *
- * The official Nmap Windows builds include the Npcap software             *
- * (https://npcap.org) for packet capture and transmission. It is under    *
- * separate license terms which forbid redistribution without special      *
- * permission. So the official Nmap Windows builds may not be              *
- * redistributed without special permission (such as an Nmap OEM           *
- * license).                                                               *
- *                                                                         *
- * Source is provided to this software because we believe users have a     *
- * right to know exactly what a program is going to do before they run it. *
- * This also allows you to audit the software for security holes.          *
- *                                                                         *
- * Source code also allows you to port Nmap to new platforms, fix bugs,    *
- * and add new features.  You are highly encouraged to submit your         *
- * changes as a Github PR or by email to the dev@nmap.org mailing list     *
- * for possible incorporation into the main distribution. Unless you       *
- * specify otherwise, it is understood that you are offering us very       *
- * broad rights to use your submissions as described in the Nmap Public    *
- * Source License Contributor Agreement. This is important because we      *
- * fund the project by selling licenses with various terms, and also       *
- * because the inability to relicense code has caused devastating          *
- * problems for other Free Software projects (such as KDE and NASM).       *
- *                                                                         *
- * The free version of Nmap is distributed in the hope that it will be     *
- * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of  *
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. Warranties,        *
- * indemnification and commercial support are all available through the    *
- * Npcap OEM program--see https://nmap.org/oem.                            *
- *                                                                         *
+ *
+ * The Nmap Security Scanner is (C) 1996-2023 Nmap Software LLC ("The Nmap
+ * Project"). Nmap is also a registered trademark of the Nmap Project.
+ *
+ * This program is distributed under the terms of the Nmap Public Source
+ * License (NPSL). The exact license text applying to a particular Nmap
+ * release or source code control revision is contained in the LICENSE
+ * file distributed with that version of Nmap or source code control
+ * revision. More Nmap copyright/legal information is available from
+ * https://nmap.org/book/man-legal.html, and further information on the
+ * NPSL license itself can be found at https://nmap.org/npsl/ . This
+ * header summarizes some key points from the Nmap license, but is no
+ * substitute for the actual license text.
+ *
+ * Nmap is generally free for end users to download and use themselves,
+ * including commercial use. It is available from https://nmap.org.
+ *
+ * The Nmap license generally prohibits companies from using and
+ * redistributing Nmap in commercial products, but we sell a special Nmap
+ * OEM Edition with a more permissive license and special features for
+ * this purpose. See https://nmap.org/oem/
+ *
+ * If you have received a written Nmap license agreement or contract
+ * stating terms other than these (such as an Nmap OEM license), you may
+ * choose to use and redistribute Nmap under those terms instead.
+ *
+ * The official Nmap Windows builds include the Npcap software
+ * (https://npcap.com) for packet capture and transmission. It is under
+ * separate license terms which forbid redistribution without special
+ * permission. So the official Nmap Windows builds may not be redistributed
+ * without special permission (such as an Nmap OEM license).
+ *
+ * Source is provided to this software because we believe users have a
+ * right to know exactly what a program is going to do before they run it.
+ * This also allows you to audit the software for security holes.
+ *
+ * Source code also allows you to port Nmap to new platforms, fix bugs, and add
+ * new features. You are highly encouraged to submit your changes as a Github PR
+ * or by email to the dev@nmap.org mailing list for possible incorporation into
+ * the main distribution. Unless you specify otherwise, it is understood that
+ * you are offering us very broad rights to use your submissions as described in
+ * the Nmap Public Source License Contributor Agreement. This is important
+ * because we fund the project by selling licenses with various terms, and also
+ * because the inability to relicense code has caused devastating problems for
+ * other Free Software projects (such as KDE and NASM).
+ *
+ * The free version of Nmap is distributed in the hope that it will be
+ * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. Warranties,
+ * indemnification and commercial support are all available through the
+ * Npcap OEM program--see https://nmap.org/oem/
+ *
  ***************************************************************************/
 
-/* $Id: nbase.h 38078 2020-10-02 16:12:22Z dmiller $ */
+/* $Id: nbase.h 38653 2023-04-14 17:11:46Z dmiller $ */
 
 #ifndef NBASE_H
 #define NBASE_H
@@ -292,7 +290,7 @@ extern "C" int vsnprintf (char *, size_t, const char *, va_list);
 #define putenv _putenv
 #define tzset _tzset
 
-#if !defined(__GNUC__)
+#if defined(_MSC_VER) && _MSC_VER < 1900
 #define snprintf _snprintf
 #endif
 
@@ -321,41 +319,37 @@ extern "C" int vsnprintf (char *, size_t, const char *, va_list);
 #define NORETURN
 #endif
 
+#ifndef WIN32
+# define CHECK_FD_OP(_Op, _Ctx) \
+  if (fd >= FD_SETSIZE) { \
+    fprintf(stderr, "Attempt to " #_Op " fd %d, which is not less than " \
+                    "FD_SETSIZE (%d). Try using a lower parallelism.", \
+                    fd, FD_SETSIZE); \
+    abort(); \
+  } \
+  _Ctx _Op(fd, fds);
+#else
+# define CHECK_FD_OP(_Op, _Ctx) _Ctx _Op(fd, fds);
+#endif
 
 static inline int checked_fd_isset(int fd, fd_set *fds) {
-#ifndef WIN32
-  if (fd >= FD_SETSIZE) {
-    fprintf(stderr, "Attempt to FD_ISSET fd %d, which is not less than "
-                    "FD_SETSIZE (%d). Try using a lower parallelism.",
-                    fd, FD_SETSIZE);
-    abort();
-  }
-#endif
-  return FD_ISSET(fd, fds);
+  CHECK_FD_OP(FD_ISSET, return);
 }
 
 static inline void checked_fd_clr(int fd, fd_set *fds) {
-#ifndef WIN32
-  if (fd >= FD_SETSIZE) {
-    fprintf(stderr, "Attempt to FD_CLR fd %d, which is not less than "
-                    "FD_SETSIZE (%d). Try using a lower parallelism.",
-                    fd, FD_SETSIZE);
-    abort();
-  }
-#endif
-  FD_CLR(fd, fds);
+  CHECK_FD_OP(FD_CLR, /**/);
 }
 
 static inline void checked_fd_set(int fd, fd_set *fds) {
-#ifndef WIN32
-  if (fd >= FD_SETSIZE) {
-    fprintf(stderr, "Attempt to FD_SET fd %d, which is not less than "
+#ifdef WIN32
+  if (fds->fd_count >= FD_SETSIZE) {
+    fprintf(stderr, "Attempt to call FD_SET, but fd_count is %d, which is not less than "
                     "FD_SETSIZE (%d). Try using a lower parallelism.",
-                    fd, FD_SETSIZE);
+                    fds->fd_count, FD_SETSIZE);
     abort();
   }
 #endif
-  FD_SET(fd, fds);
+  CHECK_FD_OP(FD_SET, /**/);
 }
 
 
@@ -427,6 +421,8 @@ int Snprintf(char *, size_t, const char *, ...)
      __attribute__ ((format (printf, 3, 4)));
 
 char *mkstr(const char *start, const char *end);
+/* Like strchr, but don't go past end. Nulls not handled specially. */
+const char *strchr_p(const char *str, const char *end, char c);
 
 int alloc_vsprintf(char **strp, const char *fmt, va_list va)
      __attribute__ ((format (printf, 2, 0)));
@@ -435,7 +431,7 @@ char *escape_windows_command_arg(const char *arg);
 
 /* parse_long is like strtol or atoi, but it allows digits only.
    No whitespace, sign, or radix prefix. */
-long parse_long(const char *s, char **tail);
+long parse_long(const char *s, const char **tail);
 
 /* This function takes a byte count and stores a short ascii equivalent
    in the supplied buffer. Eg: 0.122MB, 10.322Kb or 128B. */
