@@ -1,63 +1,61 @@
 /***************************************************************************
  * ncat_ssl.c -- SSL support functions.                                    *
  ***********************IMPORTANT NMAP LICENSE TERMS************************
- *                                                                         *
- * The Nmap Security Scanner is (C) 1996-2020 Insecure.Com LLC ("The Nmap  *
- * Project"). Nmap is also a registered trademark of the Nmap Project.     *
- *                                                                         *
- * This program is distributed under the terms of the Nmap Public Source   *
- * License (NPSL). The exact license text applying to a particular Nmap    *
- * release or source code control revision is contained in the LICENSE     *
- * file distributed with that version of Nmap or source code control       *
- * revision. More Nmap copyright/legal information is available from       *
- * https://nmap.org/book/man-legal.html, and further information on the    *
- * NPSL license itself can be found at https://nmap.org/npsl. This header  *
- * summarizes some key points from the Nmap license, but is no substitute  *
- * for the actual license text.                                            *
- *                                                                         *
- * Nmap is generally free for end users to download and use themselves,    *
- * including commercial use. It is available from https://nmap.org.        *
- *                                                                         *
- * The Nmap license generally prohibits companies from using and           *
- * redistributing Nmap in commercial products, but we sell a special Nmap  *
- * OEM Edition with a more permissive license and special features for     *
- * this purpose. See https://nmap.org/oem                                  *
- *                                                                         *
- * If you have received a written Nmap license agreement or contract       *
- * stating terms other than these (such as an Nmap OEM license), you may   *
- * choose to use and redistribute Nmap under those terms instead.          *
- *                                                                         *
- * The official Nmap Windows builds include the Npcap software             *
- * (https://npcap.org) for packet capture and transmission. It is under    *
- * separate license terms which forbid redistribution without special      *
- * permission. So the official Nmap Windows builds may not be              *
- * redistributed without special permission (such as an Nmap OEM           *
- * license).                                                               *
- *                                                                         *
- * Source is provided to this software because we believe users have a     *
- * right to know exactly what a program is going to do before they run it. *
- * This also allows you to audit the software for security holes.          *
- *                                                                         *
- * Source code also allows you to port Nmap to new platforms, fix bugs,    *
- * and add new features.  You are highly encouraged to submit your         *
- * changes as a Github PR or by email to the dev@nmap.org mailing list     *
- * for possible incorporation into the main distribution. Unless you       *
- * specify otherwise, it is understood that you are offering us very       *
- * broad rights to use your submissions as described in the Nmap Public    *
- * Source License Contributor Agreement. This is important because we      *
- * fund the project by selling licenses with various terms, and also       *
- * because the inability to relicense code has caused devastating          *
- * problems for other Free Software projects (such as KDE and NASM).       *
- *                                                                         *
- * The free version of Nmap is distributed in the hope that it will be     *
- * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of  *
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. Warranties,        *
- * indemnification and commercial support are all available through the    *
- * Npcap OEM program--see https://nmap.org/oem.                            *
- *                                                                         *
+ *
+ * The Nmap Security Scanner is (C) 1996-2023 Nmap Software LLC ("The Nmap
+ * Project"). Nmap is also a registered trademark of the Nmap Project.
+ *
+ * This program is distributed under the terms of the Nmap Public Source
+ * License (NPSL). The exact license text applying to a particular Nmap
+ * release or source code control revision is contained in the LICENSE
+ * file distributed with that version of Nmap or source code control
+ * revision. More Nmap copyright/legal information is available from
+ * https://nmap.org/book/man-legal.html, and further information on the
+ * NPSL license itself can be found at https://nmap.org/npsl/ . This
+ * header summarizes some key points from the Nmap license, but is no
+ * substitute for the actual license text.
+ *
+ * Nmap is generally free for end users to download and use themselves,
+ * including commercial use. It is available from https://nmap.org.
+ *
+ * The Nmap license generally prohibits companies from using and
+ * redistributing Nmap in commercial products, but we sell a special Nmap
+ * OEM Edition with a more permissive license and special features for
+ * this purpose. See https://nmap.org/oem/
+ *
+ * If you have received a written Nmap license agreement or contract
+ * stating terms other than these (such as an Nmap OEM license), you may
+ * choose to use and redistribute Nmap under those terms instead.
+ *
+ * The official Nmap Windows builds include the Npcap software
+ * (https://npcap.com) for packet capture and transmission. It is under
+ * separate license terms which forbid redistribution without special
+ * permission. So the official Nmap Windows builds may not be redistributed
+ * without special permission (such as an Nmap OEM license).
+ *
+ * Source is provided to this software because we believe users have a
+ * right to know exactly what a program is going to do before they run it.
+ * This also allows you to audit the software for security holes.
+ *
+ * Source code also allows you to port Nmap to new platforms, fix bugs, and add
+ * new features. You are highly encouraged to submit your changes as a Github PR
+ * or by email to the dev@nmap.org mailing list for possible incorporation into
+ * the main distribution. Unless you specify otherwise, it is understood that
+ * you are offering us very broad rights to use your submissions as described in
+ * the Nmap Public Source License Contributor Agreement. This is important
+ * because we fund the project by selling licenses with various terms, and also
+ * because the inability to relicense code has caused devastating problems for
+ * other Free Software projects (such as KDE and NASM).
+ *
+ * The free version of Nmap is distributed in the hope that it will be
+ * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. Warranties,
+ * indemnification and commercial support are all available through the
+ * Npcap OEM program--see https://nmap.org/oem/
+ *
  ***************************************************************************/
 
-/* $Id: ncat_ssl.c 38166 2020-12-24 00:35:38Z nnposter $ */
+/* $Id: ncat_ssl.c 38653 2023-04-14 17:11:46Z dmiller $ */
 
 #include "nbase.h"
 #include "ncat_config.h"
@@ -67,7 +65,6 @@
 
 #include <stdio.h>
 #include <openssl/ssl.h>
-#include <openssl/bn.h>
 #include <openssl/err.h>
 #include <openssl/rsa.h>
 #include <openssl/rand.h>
@@ -81,6 +78,14 @@
 #define FUNC_ASN1_STRING_data ASN1_STRING_data
 #endif
 
+#if OPENSSL_VERSION_NUMBER >= 0x30000000L
+#include <openssl/provider.h>
+/* Deprecated in OpenSSL 3.0 */
+#define SSL_get_peer_certificate SSL_get1_peer_certificate
+#else
+#include <openssl/bn.h>
+#endif
+
 /* Required for windows compilation to Eliminate APPLINK errors.
    See http://www.openssl.org/support/faq.html#PROG2 */
 #ifdef WIN32
@@ -98,10 +103,8 @@ enum {
 };
 #define CERTIFICATE_COMMENT "Automatically generated by Ncat. See https://nmap.org/ncat/."
 
-SSL_CTX *setup_ssl_listen(void)
+SSL_CTX *setup_ssl_listen(const SSL_METHOD *method)
 {
-    const SSL_METHOD *method;
-
     if (sslctx)
         goto done;
 
@@ -110,6 +113,17 @@ SSL_CTX *setup_ssl_listen(void)
     OpenSSL_add_all_algorithms();
     ERR_load_crypto_strings();
     SSL_load_error_strings();
+#elif OPENSSL_VERSION_NUMBER >= 0x30000000L
+  if (NULL == OSSL_PROVIDER_load(NULL, "legacy") && o.debug)
+  {
+    loguser("OpenSSL legacy provider failed to load: %s",
+        ERR_error_string(ERR_get_error(), NULL));
+  }
+  if (NULL == OSSL_PROVIDER_load(NULL, "default"))
+  {
+    loguser("OpenSSL default provider failed to load: %s",
+        ERR_error_string(ERR_get_error(), NULL));
+  }
 #endif
 
     /* RAND_status initializes the random number generator through a variety of
@@ -120,8 +134,8 @@ SSL_CTX *setup_ssl_listen(void)
     if (!RAND_status())
         bye("Failed to seed OpenSSL PRNG (RAND_status returned false).");
 
-    if (!(method = SSLv23_server_method()))
-        bye("SSLv23_server_method(): %s.", ERR_error_string(ERR_get_error(), NULL));
+    if (!method)
+        bye("Invalid SSL method: %s.", ERR_error_string(ERR_get_error(), NULL));
     if (!(sslctx = SSL_CTX_new(method)))
         bye("SSL_CTX_new(): %s.", ERR_error_string(ERR_get_error(), NULL));
 
@@ -187,27 +201,35 @@ SSL *new_ssl(int fd)
    may be a wildcard pattern. A wildcard pattern may contain only one '*', it
    must be the entire leftmost component, and there must be at least two
    components following it. len is the length of pattern; pattern may contain
-   null bytes so that len != strlen(pattern). */
-static int wildcard_match(const char *pattern, const char *hostname, size_t len)
+   null bytes so that len != strlen(pattern); pattern may also not be null terminated.
+   hostname *must* be null-terminated. */
+static int wildcard_match(const char *pattern, const char *hostname, int len)
 {
-    if (pattern[0] == '*' && pattern[1] == '.') {
+    const char *p = pattern;
+    const char *h = hostname;
+    int remaining = len;
+    if (len > 1 && pattern[0] == '*' && pattern[1] == '.') {
         /* A wildcard pattern. */
-        const char *p, *h, *dot;
+        const char *dot;
 
         /* Skip the wildcard component. */
-        p = pattern + 2;
+        p += 2;
+        remaining -= 2;
 
         /* Ensure there are no more wildcard characters. */
-        if (memchr(p, '*', len - 2) != NULL)
+        if (memchr(p, '*', remaining) != NULL)
             return 0;
 
         /* Ensure there's at least one more dot, not counting a dot at the
            end. */
-        dot = strchr(p, '.');
-        if (dot == NULL || *(dot + 1) == '\0') {
+        dot = (const char *) memchr(p, '.', remaining);
+        if (dot == NULL /* not found */
+          || dot - p == remaining /* dot in last position */
+          || *(dot + 1) == '\0') /* dot immediately before null terminator */
+          {
             if (o.debug > 1) {
-                logdebug("Wildcard name \"%s\" doesn't have at least two"
-                    " components after the wildcard; rejecting.\n", pattern);
+                logdebug("Wildcard name \"%.*s\" doesn't have at least two"
+                    " components after the wildcard; rejecting.\n", len, pattern);
             }
             return 0;
         }
@@ -218,14 +240,12 @@ static int wildcard_match(const char *pattern, const char *hostname, size_t len)
             return 0;
         h++;
 
-        /* Compare what remains of the pattern and hostname. */
-        return len == strlen(h) + (p - pattern) && strcmp(p, h) == 0;
-    } else {
-        /* Normal string comparison. Check the name length because I'm concerned
-           about someone somehow embedding a '\0' in the subject and matching
-           against a shorter name. */
-        return len == strlen(hostname) && strcmp(pattern, hostname) == 0;
     }
+    /* Compare what remains of the pattern and hostname. */
+    /* Normal string comparison. Check the name length because I'm concerned
+       about someone somehow embedding a '\0' in the subject and matching
+       against a shorter name. */
+    return remaining == strlen(h) && strncmp(p, h, remaining) == 0;
 }
 
 /* Match a hostname against the contents of a dNSName field of the
@@ -300,11 +320,13 @@ static int cert_match_dnsname(X509 *cert, const char *hostname,
 
         gen_name = sk_GENERAL_NAME_value(gen_names, i);
         if (gen_name->type == GEN_DNS) {
+            const char *dnsname = (const char *) FUNC_ASN1_STRING_data(gen_name->d.dNSName);
+            int dnslen = ASN1_STRING_length(gen_name->d.dNSName);
             if (o.debug > 1)
-                logdebug("Checking certificate DNS name \"%s\" against \"%s\".\n", FUNC_ASN1_STRING_data(gen_name->d.dNSName), hostname);
+                logdebug("Checking certificate DNS name \"%.*s\" against \"%s\".\n", dnslen, dnsname, hostname);
             if (num_checked != NULL)
                 (*num_checked)++;
-            if (wildcard_match((char *) FUNC_ASN1_STRING_data(gen_name->d.dNSName), hostname, ASN1_STRING_length(gen_name->d.dNSName)))
+            if (wildcard_match(dnsname, hostname, dnslen))
                 return 1;
         }
     }
@@ -447,14 +469,16 @@ int ssl_post_connect_check(SSL *ssl, const char *hostname)
    "Making Certificates"; and apps/req.c in the OpenSSL source. */
 static int ssl_gen_cert(X509 **cert, EVP_PKEY **key)
 {
-    RSA *rsa = NULL;
     X509_NAME *subj;
     X509_EXTENSION *ext;
     X509V3_CTX ctx;
-    BIGNUM *bne = NULL;
     const char *commonName = "localhost";
     char dNSName[128];
-    int rc, ret=0;
+    int rc;
+#if OPENSSL_VERSION_NUMBER < 0x30000000L
+    int ret = 0;
+    RSA *rsa = NULL;
+    BIGNUM *bne = NULL;
 
     *cert = NULL;
     *key = NULL;
@@ -483,6 +507,12 @@ static int ssl_gen_cert(X509 **cert, EVP_PKEY **key)
         RSA_free(rsa);
         goto err;
     }
+#else
+    *cert = NULL;
+    *key = EVP_RSA_gen(DEFAULT_KEY_BITS);
+    if (*key == NULL)
+        goto err;
+#endif
 
     /* Generate a certificate. */
     *cert = X509_new();
