@@ -1,131 +1,61 @@
 /***************************************************************************
  * util.c -- Various utility functions.                                    *
  ***********************IMPORTANT NMAP LICENSE TERMS************************
- *                                                                         *
- * The Nmap Security Scanner is (C) 1996-2017 Insecure.Com LLC ("The Nmap  *
- * Project"). Nmap is also a registered trademark of the Nmap Project.     *
- * This program is free software; you may redistribute and/or modify it    *
- * under the terms of the GNU General Public License as published by the   *
- * Free Software Foundation; Version 2 ("GPL"), BUT ONLY WITH ALL OF THE   *
- * CLARIFICATIONS AND EXCEPTIONS DESCRIBED HEREIN.  This guarantees your   *
- * right to use, modify, and redistribute this software under certain      *
- * conditions.  If you wish to embed Nmap technology into proprietary      *
- * software, we sell alternative licenses (contact sales@nmap.com).        *
- * Dozens of software vendors already license Nmap technology such as      *
- * host discovery, port scanning, OS detection, version detection, and     *
- * the Nmap Scripting Engine.                                              *
- *                                                                         *
- * Note that the GPL places important restrictions on "derivative works",  *
- * yet it does not provide a detailed definition of that term.  To avoid   *
- * misunderstandings, we interpret that term as broadly as copyright law   *
- * allows.  For example, we consider an application to constitute a        *
- * derivative work for the purpose of this license if it does any of the   *
- * following with any software or content covered by this license          *
- * ("Covered Software"):                                                   *
- *                                                                         *
- * o Integrates source code from Covered Software.                         *
- *                                                                         *
- * o Reads or includes copyrighted data files, such as Nmap's nmap-os-db   *
- * or nmap-service-probes.                                                 *
- *                                                                         *
- * o Is designed specifically to execute Covered Software and parse the    *
- * results (as opposed to typical shell or execution-menu apps, which will *
- * execute anything you tell them to).                                     *
- *                                                                         *
- * o Includes Covered Software in a proprietary executable installer.  The *
- * installers produced by InstallShield are an example of this.  Including *
- * Nmap with other software in compressed or archival form does not        *
- * trigger this provision, provided appropriate open source decompression  *
- * or de-archiving software is widely available for no charge.  For the    *
- * purposes of this license, an installer is considered to include Covered *
- * Software even if it actually retrieves a copy of Covered Software from  *
- * another source during runtime (such as by downloading it from the       *
- * Internet).                                                              *
- *                                                                         *
- * o Links (statically or dynamically) to a library which does any of the  *
- * above.                                                                  *
- *                                                                         *
- * o Executes a helper program, module, or script to do any of the above.  *
- *                                                                         *
- * This list is not exclusive, but is meant to clarify our interpretation  *
- * of derived works with some common examples.  Other people may interpret *
- * the plain GPL differently, so we consider this a special exception to   *
- * the GPL that we apply to Covered Software.  Works which meet any of     *
- * these conditions must conform to all of the terms of this license,      *
- * particularly including the GPL Section 3 requirements of providing      *
- * source code and allowing free redistribution of the work as a whole.    *
- *                                                                         *
- * As another special exception to the GPL terms, the Nmap Project grants  *
- * permission to link the code of this program with any version of the     *
- * OpenSSL library which is distributed under a license identical to that  *
- * listed in the included docs/licenses/OpenSSL.txt file, and distribute   *
- * linked combinations including the two.                                  *
- *                                                                         *
- * The Nmap Project has permission to redistribute Npcap, a packet         *
- * capturing driver and library for the Microsoft Windows platform.        *
- * Npcap is a separate work with it's own license rather than this Nmap    *
- * license.  Since the Npcap license does not permit redistribution        *
- * without special permission, our Nmap Windows binary packages which      *
- * contain Npcap may not be redistributed without special permission.      *
- *                                                                         *
- * Any redistribution of Covered Software, including any derived works,    *
- * must obey and carry forward all of the terms of this license, including *
- * obeying all GPL rules and restrictions.  For example, source code of    *
- * the whole work must be provided and free redistribution must be         *
- * allowed.  All GPL references to "this License", are to be treated as    *
- * including the terms and conditions of this license text as well.        *
- *                                                                         *
- * Because this license imposes special exceptions to the GPL, Covered     *
- * Work may not be combined (even as part of a larger work) with plain GPL *
- * software.  The terms, conditions, and exceptions of this license must   *
- * be included as well.  This license is incompatible with some other open *
- * source licenses as well.  In some cases we can relicense portions of    *
- * Nmap or grant special permissions to use it in other open source        *
- * software.  Please contact fyodor@nmap.org with any such requests.       *
- * Similarly, we don't incorporate incompatible open source software into  *
- * Covered Software without special permission from the copyright holders. *
- *                                                                         *
- * If you have any questions about the licensing restrictions on using     *
- * Nmap in other works, are happy to help.  As mentioned above, we also    *
- * offer alternative license to integrate Nmap into proprietary            *
- * applications and appliances.  These contracts have been sold to dozens  *
- * of software vendors, and generally include a perpetual license as well  *
- * as providing for priority support and updates.  They also fund the      *
- * continued development of Nmap.  Please email sales@nmap.com for further *
- * information.                                                            *
- *                                                                         *
- * If you have received a written license agreement or contract for        *
- * Covered Software stating terms other than these, you may choose to use  *
- * and redistribute Covered Software under those terms instead of these.   *
- *                                                                         *
- * Source is provided to this software because we believe users have a     *
- * right to know exactly what a program is going to do before they run it. *
- * This also allows you to audit the software for security holes.          *
- *                                                                         *
- * Source code also allows you to port Nmap to new platforms, fix bugs,    *
- * and add new features.  You are highly encouraged to send your changes   *
- * to the dev@nmap.org mailing list for possible incorporation into the    *
- * main distribution.  By sending these changes to Fyodor or one of the    *
- * Insecure.Org development mailing lists, or checking them into the Nmap  *
- * source code repository, it is understood (unless you specify            *
- * otherwise) that you are offering the Nmap Project the unlimited,        *
- * non-exclusive right to reuse, modify, and relicense the code.  Nmap     *
- * will always be available Open Source, but this is important because     *
- * the inability to relicense code has caused devastating problems for     *
- * other Free Software projects (such as KDE and NASM).  We also           *
- * occasionally relicense the code to third parties as discussed above.    *
- * If you wish to specify special license conditions of your               *
- * contributions, just say so when you send them.                          *
- *                                                                         *
- * This program is distributed in the hope that it will be useful, but     *
- * WITHOUT ANY WARRANTY; without even the implied warranty of              *
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the Nmap      *
- * license file for more details (it's in a COPYING file included with     *
- * Nmap, and also available from https://svn.nmap.org/nmap/COPYING)        *
- *                                                                         *
+ *
+ * The Nmap Security Scanner is (C) 1996-2023 Nmap Software LLC ("The Nmap
+ * Project"). Nmap is also a registered trademark of the Nmap Project.
+ *
+ * This program is distributed under the terms of the Nmap Public Source
+ * License (NPSL). The exact license text applying to a particular Nmap
+ * release or source code control revision is contained in the LICENSE
+ * file distributed with that version of Nmap or source code control
+ * revision. More Nmap copyright/legal information is available from
+ * https://nmap.org/book/man-legal.html, and further information on the
+ * NPSL license itself can be found at https://nmap.org/npsl/ . This
+ * header summarizes some key points from the Nmap license, but is no
+ * substitute for the actual license text.
+ *
+ * Nmap is generally free for end users to download and use themselves,
+ * including commercial use. It is available from https://nmap.org.
+ *
+ * The Nmap license generally prohibits companies from using and
+ * redistributing Nmap in commercial products, but we sell a special Nmap
+ * OEM Edition with a more permissive license and special features for
+ * this purpose. See https://nmap.org/oem/
+ *
+ * If you have received a written Nmap license agreement or contract
+ * stating terms other than these (such as an Nmap OEM license), you may
+ * choose to use and redistribute Nmap under those terms instead.
+ *
+ * The official Nmap Windows builds include the Npcap software
+ * (https://npcap.com) for packet capture and transmission. It is under
+ * separate license terms which forbid redistribution without special
+ * permission. So the official Nmap Windows builds may not be redistributed
+ * without special permission (such as an Nmap OEM license).
+ *
+ * Source is provided to this software because we believe users have a
+ * right to know exactly what a program is going to do before they run it.
+ * This also allows you to audit the software for security holes.
+ *
+ * Source code also allows you to port Nmap to new platforms, fix bugs, and add
+ * new features. You are highly encouraged to submit your changes as a Github PR
+ * or by email to the dev@nmap.org mailing list for possible incorporation into
+ * the main distribution. Unless you specify otherwise, it is understood that
+ * you are offering us very broad rights to use your submissions as described in
+ * the Nmap Public Source License Contributor Agreement. This is important
+ * because we fund the project by selling licenses with various terms, and also
+ * because the inability to relicense code has caused devastating problems for
+ * other Free Software projects (such as KDE and NASM).
+ *
+ * The free version of Nmap is distributed in the hope that it will be
+ * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. Warranties,
+ * indemnification and commercial support are all available through the
+ * Npcap OEM program--see https://nmap.org/oem/
+ *
  ***************************************************************************/
 
-/* $Id: util.c 36893 2017-07-30 04:09:48Z dmiller $ */
+/* $Id: util.c 38653 2023-04-14 17:11:46Z dmiller $ */
 
 #include "sys_wrap.h"
 #include "util.h"
@@ -140,6 +70,7 @@
 #include <stdlib.h>
 #include <stdarg.h>
 #include <string.h>
+#include <stddef.h>
 
 #if HAVE_SYS_STAT_H
 #include <sys/stat.h>
@@ -151,6 +82,10 @@
 #include <unistd.h>
 #endif
 
+#if HAVE_LINUX_VM_SOCKETS_H
+#include <linux/vm_sockets.h>
+#endif
+
 /* safely add 2 size_t */
 size_t sadd(size_t l, size_t r)
 {
@@ -235,7 +170,18 @@ void logtest(const char *fmt, ...)
 /* Exit status 2 indicates a program error other than a network error. */
 void die(char *err)
 {
+#ifdef WIN32
+  int error_number;
+  char *strerror_s;
+  error_number = GetLastError();
+  FormatMessage(FORMAT_MESSAGE_ALLOCATE_BUFFER|FORMAT_MESSAGE_FROM_SYSTEM,
+      NULL, error_number, MAKELANGID (LANG_NEUTRAL, SUBLANG_DEFAULT),
+      (LPTSTR) &strerror_s,  0, NULL);
+  fprintf(stderr, "%s: %s\n", err, strerror_s);
+  HeapFree(GetProcessHeap(), 0, strerror_s);
+#else
     perror(err);
+#endif
     fflush(stderr);
     exit(2);
 }
@@ -369,6 +315,65 @@ int addr_is_local(const union sockaddr_u *su)
     }
 }
 
+/* Converts a sockaddr_u to a string representation. Since a static buffer is
+ * returned, this is not thread-safe and can only be used once in calls like
+ * printf(). ss_len may be 0 if it is not already known.
+*/
+const char *socktop(const union sockaddr_u *su, socklen_t ss_len)
+{
+    static char buf[INET6_ADDRSTRLEN + sizeof(union sockaddr_u)];
+    size_t size = sizeof(buf);
+
+    switch (su->storage.ss_family) {
+#if HAVE_SYS_UN_H
+        case AF_UNIX:
+            ncat_assert(ss_len <= sizeof(struct sockaddr_un));
+            if (ss_len == sizeof(sa_family_t)) {
+                /* Unnamed socket */
+                Strncpy(buf, "(unnamed socket)", sizeof(buf));
+            }
+            else {
+                if (ss_len < sizeof(sa_family_t)) {
+                    /* socket path not guaranteed to be valid, but we'll try. */
+                    size = sizeof(su->un.sun_path);
+                }
+                else {
+                    /* We will add null terminator at size + 1 in case it was missing. */
+                    size = MIN(sizeof(buf) - 1,
+                            ss_len - offsetof(struct sockaddr_un, sun_path));
+                }
+                if (su->un.sun_path[0] == '\0') {
+                    /* Abstract socket (Linux extension) */
+                    memcpy(buf, su->un.sun_path + 1, size - 1);
+                    Strncpy(buf + size, " (abstract socket)", sizeof(buf) - size);
+                }
+                else {
+                    memcpy(buf, su->un.sun_path, size);
+                    buf[size+1] = '\0';
+                }
+                /* In case we got junk data, make it safe. */
+                replacenonprintable(buf, strlen(buf), '?');
+            }
+            break;
+#endif
+#ifdef HAVE_LINUX_VM_SOCKETS_H
+        case AF_VSOCK:
+            Snprintf(buf, sizeof(buf), "%u:%u", su->vm.svm_cid, su->vm.svm_port);
+            break;
+#endif
+        case AF_INET:
+            Snprintf(buf, sizeof(buf), "%s:%hu", inet_socktop(su), inet_port(su));
+            break;
+        case AF_INET6:
+            Snprintf(buf, sizeof(buf), "[%s]:%hu", inet_socktop(su), inet_port(su));
+            break;
+        default:
+            return NULL;
+            break;
+    }
+    return buf;
+}
+
 /* Converts an IP address given in a sockaddr_u to an IPv4 or
    IPv6 IP address string.  Since a static buffer is returned, this is
    not thread-safe and can only be used once in calls like printf()
@@ -385,7 +390,7 @@ const char *inet_socktop(const union sockaddr_u *su)
         addr = (void *) &su->in6.sin6_addr;
 #endif
     else
-        addr = NULL;
+        bye("Invalid address family passed to inet_socktop().");
 
     if (inet_ntop(su->storage.ss_family, addr, buf, sizeof(buf)) == NULL) {
         bye("Failed to convert address to presentation format!  Error: %s.",
@@ -398,14 +403,19 @@ const char *inet_socktop(const union sockaddr_u *su)
 /* Returns the port number in HOST BYTE ORDER based on the su's family */
 unsigned short inet_port(const union sockaddr_u *su)
 {
-    if (su->storage.ss_family == AF_INET)
-        return ntohs(su->in.sin_port);
+    switch (su->storage.ss_family) {
+        case AF_INET:
+            return ntohs(su->in.sin_port);
+            break;
 #if HAVE_IPV6
-    else if (su->storage.ss_family == AF_INET6)
-        return ntohs(su->in6.sin6_port);
+        case AF_INET6:
+            return ntohs(su->in6.sin6_port);
+            break;
 #endif
-
-    bye("Invalid address family passed to inet_port().");
+        default:
+            bye("Invalid address family passed to inet_port().");
+            break;
+    }
     return 0;
 }
 
@@ -447,52 +457,18 @@ int do_listen(int type, int proto, const union sockaddr_u *srcaddr_u)
 #endif
 #endif
 
-    switch(srcaddr_u->storage.ss_family) {
-#ifdef HAVE_SYS_UN_H
-      case AF_UNIX:
-        sa_len = SUN_LEN(&srcaddr_u->un);
-        break;
-#endif
-#ifdef HAVE_SOCKADDR_SA_LEN
-      default:
-        sa_len = srcaddr_u->sockaddr.sa_len;
-        break;
-#else
-      case AF_INET:
-        sa_len = sizeof (struct sockaddr_in);
-        break;
-#ifdef AF_INET6
-      case AF_INET6:
-        sa_len = sizeof (struct sockaddr_in6);
-        break;
-#endif
-      default:
-        sa_len = sizeof(*srcaddr_u);
-        break;
-#endif
-    }
+    sa_len = get_socklen(srcaddr_u);
 
     if (bind(sock, &srcaddr_u->sockaddr, sa_len) < 0) {
-#ifdef HAVE_SYS_UN_H
-        if (srcaddr_u->storage.ss_family == AF_UNIX)
-            bye("bind to %s: %s.", srcaddr_u->un.sun_path,
+        bye("bind to %s: %s.", socktop(srcaddr_u, sa_len),
                 socket_strerror(socket_errno()));
-        else
-#endif
-            bye("bind to %s:%hu: %s.", inet_socktop(srcaddr_u),
-                inet_port(srcaddr_u), socket_strerror(socket_errno()));
     }
 
     if (type == SOCK_STREAM)
         Listen(sock, BACKLOG);
 
     if (o.verbose) {
-#ifdef HAVE_SYS_UN_H
-        if (srcaddr_u->storage.ss_family == AF_UNIX)
-            loguser("Listening on %s\n", srcaddr_u->un.sun_path);
-        else
-#endif
-            loguser("Listening on %s:%hu\n", inet_socktop(srcaddr_u), inet_port(srcaddr_u));
+        loguser("Listening on %s\n", socktop(srcaddr_u, sa_len));
     }
     if (o.test)
         logtest("LISTEN\n");
@@ -500,6 +476,8 @@ int do_listen(int type, int proto, const union sockaddr_u *srcaddr_u)
     return sock;
 }
 
+/* Only used in proxy connect functions, so doesn't need to support address
+ * families that don't support proxying like AF_UNIX and AF_VSOCK */
 int do_connect(int type)
 {
     int sock = 0;
@@ -513,16 +491,11 @@ int do_connect(int type)
     sock = inheritable_socket(targetaddrs->addr.storage.ss_family, type, 0);
 
     if (srcaddr.storage.ss_family != AF_UNSPEC) {
-        size_t sa_len;
+        size_t sa_len = get_socklen(&srcaddr);
 
-#ifdef HAVE_SOCKADDR_SA_LEN
-        sa_len = srcaddr.sockaddr.sa_len;
-#else
-        sa_len = sizeof(srcaddr);
-#endif
         if (bind(sock, &srcaddr.sockaddr, sa_len) < 0) {
-            bye("bind to %s:%hu: %s.", inet_socktop(&srcaddr),
-                inet_port(&srcaddr), socket_strerror(socket_errno()));
+            bye("bind to %s: %s.", socktop(&srcaddr, sa_len),
+                    socket_strerror(socket_errno()));
         }
     }
 
@@ -570,9 +543,9 @@ int allow_access(const union sockaddr_u *su)
 {
     /* A host not in the allow set is denied, but only if the --allow or
        --allowfile option was given. */
-    if (o.allow && !addrset_contains(&o.allowset, &su->sockaddr))
+    if (o.allow && !addrset_contains(o.allowset, &su->sockaddr))
         return 0;
-    if (addrset_contains(&o.denyset, &su->sockaddr))
+    if (addrset_contains(o.denyset, &su->sockaddr))
         return 0;
 
     return 1;
@@ -629,32 +602,42 @@ int add_fd(fd_list_t *fdl, int fd)
 int rm_fd(fd_list_t *fdl, int fd)
 {
     int x = 0, last = fdl->nfds;
+    int found = -1;
+    int newfdmax = 0;
 
     /* make sure we have a list */
     if (last == 0)
         bye("Program bug: Trying to remove fd from list with no fds.");
 
     /* find the fd in the list */
-    for (x = 0; x < last; x++)
-        if (fdl->fds[x].fd == fd)
-            break;
+    for (x = 0; x < last; x++) {
+        struct fdinfo *fdi = &fdl->fds[x];
+        if (fdi->fd == fd) {
+            found = x;
+            /* If it's not the max, we can bail early. */
+            if (fd < fdl->fdmax) {
+                newfdmax = fdl->fdmax;
+                break;
+            }
+        }
+        else if (fdi->fd > newfdmax)
+            newfdmax = fdi->fd;
+    }
+    fdl->fdmax = newfdmax;
 
     /* make sure we found it */
-    if (x == last)
+    if (found < 0)
         bye("Program bug: fd (%d) not on list.", fd);
 
     /* remove it, does nothing if (last == 1) */
     if (o.debug > 1)
         logdebug("Swapping fd[%d] (%d) with fd[%d] (%d)\n",
-                 x, fdl->fds[x].fd, last - 1, fdl->fds[last - 1].fd);
-    fdl->fds[x] = fdl->fds[last - 1];
+                 found, fdl->fds[found].fd, last - 1, fdl->fds[last - 1].fd);
+    fdl->fds[found] = fdl->fds[last - 1];
+    fdl->state++;
 
     fdl->nfds--;
 
-    /* was it the max */
-    if (fd == fdl->fdmax)
-        fdl->fdmax = get_maxfd(fdl);
-
     if (o.debug > 1)
         logdebug("Removed fd %d from list, nfds %d, maxfd %d\n", fd, fdl->nfds, fdl->fdmax);
     return 0;
@@ -689,6 +672,7 @@ void init_fdlist(fd_list_t *fdl, int maxfds)
     fdl->nfds = 0;
     fdl->fdmax = -1;
     fdl->maxfds = maxfds;
+    fdl->state = 0;
 
     if (o.debug > 1)
         logdebug("Initialized fdlist with %d maxfds\n", maxfds);
@@ -699,6 +683,7 @@ void free_fdlist(fd_list_t *fdl)
     free(fdl->fds);
     fdl->nfds = 0;
     fdl->fdmax = -1;
+    fdl->state = 0;
 }
 
 
