@@ -3,57 +3,53 @@
  * the nsock parallel socket event library                                 *
  *                                                                         *
  ***********************IMPORTANT NSOCK LICENSE TERMS***********************
- *                                                                         *
- * The nsock parallel socket event library is (C) 1999-2017 Insecure.Com   *
- * LLC This library is free software; you may redistribute and/or          *
- * modify it under the terms of the GNU General Public License as          *
- * published by the Free Software Foundation; Version 2.  This guarantees  *
- * your right to use, modify, and redistribute this software under certain *
- * conditions.  If this license is unacceptable to you, Insecure.Com LLC   *
- * may be willing to sell alternative licenses (contact                    *
- * sales@insecure.com ).                                                   *
- *                                                                         *
- * As a special exception to the GPL terms, Insecure.Com LLC grants        *
- * permission to link the code of this program with any version of the     *
- * OpenSSL library which is distributed under a license identical to that  *
- * listed in the included docs/licenses/OpenSSL.txt file, and distribute   *
- * linked combinations including the two. You must obey the GNU GPL in all *
- * respects for all of the code used other than OpenSSL.  If you modify    *
- * this file, you may extend this exception to your version of the file,   *
- * but you are not obligated to do so.                                     *
- *                                                                         *
- * If you received these files with a written license agreement stating    *
- * terms other than the (GPL) terms above, then that alternative license   *
- * agreement takes precedence over this comment.                           *
- *                                                                         *
- * Source is provided to this software because we believe users have a     *
- * right to know exactly what a program is going to do before they run it. *
- * This also allows you to audit the software for security holes.          *
- *                                                                         *
- * Source code also allows you to port Nmap to new platforms, fix bugs,    *
- * and add new features.  You are highly encouraged to send your changes   *
- * to the dev@nmap.org mailing list for possible incorporation into the    *
- * main distribution.  By sending these changes to Fyodor or one of the    *
- * Insecure.Org development mailing lists, or checking them into the Nmap  *
- * source code repository, it is understood (unless you specify otherwise) *
- * that you are offering the Nmap Project (Insecure.Com LLC) the           *
- * unlimited, non-exclusive right to reuse, modify, and relicense the      *
- * code.  Nmap will always be available Open Source, but this is important *
- * because the inability to relicense code has caused devastating problems *
- * for other Free Software projects (such as KDE and NASM).  We also       *
- * occasionally relicense the code to third parties as discussed above.    *
- * If you wish to specify special license conditions of your               *
- * contributions, just say so when you send them.                          *
- *                                                                         *
- * This program is distributed in the hope that it will be useful, but     *
- * WITHOUT ANY WARRANTY; without even the implied warranty of              *
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
- * General Public License v2.0 for more details                            *
- * (http://www.gnu.org/licenses/gpl-2.0.html).                             *
- *                                                                         *
+ *
+ * The nsock parallel socket event library is (C) 1999-2023 Nmap Software LLC
+ * This library is free software; you may redistribute and/or modify it under
+ * the terms of the GNU General Public License as published by the Free Software
+ * Foundation; Version 2. This guarantees your right to use, modify, and
+ * redistribute this software under certain conditions. If this license is
+ * unacceptable to you, Nmap Software LLC may be willing to sell alternative
+ * licenses (contact sales@nmap.com ).
+ *
+ * As a special exception to the GPL terms, Nmap Software LLC grants permission
+ * to link the code of this program with any version of the OpenSSL library
+ * which is distributed under a license identical to that listed in the included
+ * docs/licenses/OpenSSL.txt file, and distribute linked combinations including
+ * the two. You must obey the GNU GPL in all respects for all of the code used
+ * other than OpenSSL. If you modify this file, you may extend this exception to
+ * your version of the file, but you are not obligated to do so.
+ *
+ * If you received these files with a written license agreement stating terms
+ * other than the (GPL) terms above, then that alternative license agreement
+ * takes precedence over this comment.
+ *
+ * Source is provided to this software because we believe users have a right to
+ * know exactly what a program is going to do before they run it. This also
+ * allows you to audit the software for security holes.
+ *
+ * Source code also allows you to port Nmap to new platforms, fix bugs, and add
+ * new features. You are highly encouraged to send your changes to the
+ * dev@nmap.org mailing list for possible incorporation into the main
+ * distribution. By sending these changes to Fyodor or one of the Insecure.Org
+ * development mailing lists, or checking them into the Nmap source code
+ * repository, it is understood (unless you specify otherwise) that you are
+ * offering the Nmap Project (Nmap Software LLC) the unlimited, non-exclusive
+ * right to reuse, modify, and relicense the code. Nmap will always be available
+ * Open Source, but this is important because the inability to relicense code
+ * has caused devastating problems for other Free Software projects (such as KDE
+ * and NASM). We also occasionally relicense the code to third parties as
+ * discussed above. If you wish to specify special license conditions of your
+ * contributions, just say so when you send them.
+ *
+ * This program is distributed in the hope that it will be useful, but WITHOUT
+ * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
+ * FOR A PARTICULAR PURPOSE. See the GNU General Public License v2.0 for more
+ * details (http://www.gnu.org/licenses/gpl-2.0.html).
+ *
  ***************************************************************************/
 
-/* $Id: nsock_pcap.c 36788 2017-06-07 12:32:38Z dmiller $ */
+/* $Id: nsock_pcap.c 38653 2023-04-14 17:11:46Z dmiller $ */
 
 #include "nsock.h"
 #include "nsock_internal.h"
@@ -81,6 +77,11 @@ extern struct timeval nsock_tod;
 
 #if HAVE_PCAP
 
+#ifndef PCAP_NETMASK_UNKNOWN
+/* libpcap before 1.1.1 (e.g. WinPcap) doesn't handle this specially, so just use 0 netmask */
+#define PCAP_NETMASK_UNKNOWN 0
+#endif
+
 #define PCAP_OPEN_MAX_RETRIES   3
 
 #define PCAP_FAILURE_EXPL_MESSAGE  \
@@ -197,18 +198,6 @@ static int nsock_pcap_get_l3_offset(pcap_t *pt, int *dl) {
   return (offset);
 }
 
-static int nsock_pcap_try_open(struct npool *nsp, mspcap *mp, const char *dev,
-                               int snaplen, int promisc, int timeout_ms,
-                               char *errbuf) {
-    mp->pt = pcap_open_live(dev, snaplen, promisc, timeout_ms, errbuf);
-    if (!mp->pt) {
-      nsock_log_error("pcap_open_live(%s, %d, %d, %d) failed with error: %s",
-                      dev, snaplen, promisc, timeout_ms, errbuf);
-      return -1;
-    }
-    return 0;
-}
-
 /* Convert new nsiod to pcap descriptor. Other parameters have
  * the same meaning as for pcap_open_live in pcap(3).
  *   device   : pcap-style device name
@@ -265,33 +254,60 @@ int nsock_pcap_open(nsock_pool nsp, nsock_iod nsiod, const char *pcap_device,
                  "(promisc=%i snaplen=%i to_ms=%i) (IOD #%li)",
                  pcap_device,bpf, promisc, snaplen, to_ms, nsi->id);
 
-  failed = 0;
-  do {
-    rc = nsock_pcap_try_open(ms, mp, pcap_device, snaplen, promisc, to_ms, errbuf);
-    if (rc) {
-      failed++;
-      nsock_log_error("Will wait %d seconds then retry.", 4 * failed);
-      sleep(4 * failed);
-    }
-  } while (rc && failed < PCAP_OPEN_MAX_RETRIES);
-
-  if (rc) {
-    nsock_log_error("pcap_open_live(%s, %d, %d, %d) failed %d times.",
-                    pcap_device, snaplen, promisc, to_ms, failed);
+#ifdef __amigaos__
+  // Amiga doesn't have pcap_create
+  // TODO: Does Nmap still work on Amiga?
+  mp->pt = pcap_open_live(pcap_device, snaplen, promisc, to_ms, errbuf);
+  if (!mp->pt) {
+    nsock_log_error("pcap_open_live(%s, %d, %d, %d) failed with error: %s",
+        pcap_device, snaplen, promisc, to_ms, errbuf);
+    nsock_log_error(PCAP_FAILURE_EXPL_MESSAGE);
+    nsock_log_error("Can't open pcap! Are you root?");
+    return -1;
+  }
+#else
+  mp->pt = pcap_create(pcap_device, errbuf);
+  if (!mp->pt) {
+    nsock_log_error("pcap_create(%s) failed with error: %s", pcap_device, errbuf);
     nsock_log_error(PCAP_FAILURE_EXPL_MESSAGE);
     nsock_log_error("Can't open pcap! Are you root?");
     return -1;
   }
 
+#define MY_PCAP_SET(func, p_t, val) do {\
+  failed = func(p_t, val);\
+  if (failed) {\
+    nsock_log_error(#func "(%d) FAILED: %d.", val, failed);\
+    pcap_close(p_t);\
+    return -1;\
+  }\
+} while(0);
+
+  MY_PCAP_SET(pcap_set_snaplen, mp->pt, snaplen);
+  MY_PCAP_SET(pcap_set_promisc, mp->pt, promisc);
+  MY_PCAP_SET(pcap_set_timeout, mp->pt, to_ms);
+#ifdef HAVE_PCAP_SET_IMMEDIATE_MODE
+  MY_PCAP_SET(pcap_set_immediate_mode, mp->pt, 1);
+#endif
+
+  failed = pcap_activate(mp->pt);
+  if (failed < 0) {
+    // PCAP error
+    nsock_log_error("pcap_activate(%s) FAILED: %s.", pcap_device, pcap_geterr(mp->pt));
+    pcap_close(mp->pt);
+    mp->pt = NULL;
+    return -1;
+  }
+  else if (failed > 0) {
+    // PCAP warning, report but assume it'll still work
+    nsock_log_error("pcap_activate(%s) WARNING: %s.", pcap_device, pcap_geterr(mp->pt));
+  }
+#endif /* not __amigaos__ */
+
   rc = nsock_pcap_set_filter(ms, mp->pt, pcap_device, bpf);
   if (rc)
     return rc;
 
-#ifdef WIN32
-  /* We want any responses back ASAP */
-  pcap_setmintocopy(mp->pt, 1);
-#endif
-
   mp->l3_offset = nsock_pcap_get_l3_offset(mp->pt, &datalink);
   mp->snaplen = snaplen;
   mp->datalink = datalink;
@@ -303,18 +319,24 @@ int nsock_pcap_open(nsock_pool nsp, nsock_iod nsiod, const char *pcap_device,
 #endif
   mp->readsd_count = 0;
 
+#ifndef HAVE_PCAP_SET_IMMEDIATE_MODE
+  /* This is already handled by pcap_set_immediate_mode if available */
+#ifdef BIOCIMMEDIATE
   /* Without setting this ioctl, some systems (BSDs, though it depends on the
    * release) will buffer packets in non-blocking mode and only return them in a
    * bunch when the buffer is full. Setting the ioctl makes each one be
    * delivered immediately. This is how Linux works by default. See the comments
    * surrounding the setting of BIOCIMMEDIATE in libpcap/pcap-bpf.c. */
-#ifdef BIOCIMMEDIATE
   if (mp->pcap_desc != -1) {
     int immediate = 1;
 
     if (ioctl(mp->pcap_desc, BIOCIMMEDIATE, &immediate) < 0)
       fatal("Cannot set BIOCIMMEDIATE on pcap descriptor");
   }
+#elif defined WIN32
+  /* We want any responses back ASAP */
+  pcap_setmintocopy(mp->pt, 0);
+#endif
 #endif
 
   /* Set device non-blocking */
