@@ -2,60 +2,58 @@
  * scan_lists.cc -- Structures and functions for lists of ports to scan    *
  * and scan types                                                          *
  ***********************IMPORTANT NMAP LICENSE TERMS************************
- *                                                                         *
- * The Nmap Security Scanner is (C) 1996-2020 Insecure.Com LLC ("The Nmap  *
- * Project"). Nmap is also a registered trademark of the Nmap Project.     *
- *                                                                         *
- * This program is distributed under the terms of the Nmap Public Source   *
- * License (NPSL). The exact license text applying to a particular Nmap    *
- * release or source code control revision is contained in the LICENSE     *
- * file distributed with that version of Nmap or source code control       *
- * revision. More Nmap copyright/legal information is available from       *
- * https://nmap.org/book/man-legal.html, and further information on the    *
- * NPSL license itself can be found at https://nmap.org/npsl. This header  *
- * summarizes some key points from the Nmap license, but is no substitute  *
- * for the actual license text.                                            *
- *                                                                         *
- * Nmap is generally free for end users to download and use themselves,    *
- * including commercial use. It is available from https://nmap.org.        *
- *                                                                         *
- * The Nmap license generally prohibits companies from using and           *
- * redistributing Nmap in commercial products, but we sell a special Nmap  *
- * OEM Edition with a more permissive license and special features for     *
- * this purpose. See https://nmap.org/oem                                  *
- *                                                                         *
- * If you have received a written Nmap license agreement or contract       *
- * stating terms other than these (such as an Nmap OEM license), you may   *
- * choose to use and redistribute Nmap under those terms instead.          *
- *                                                                         *
- * The official Nmap Windows builds include the Npcap software             *
- * (https://npcap.org) for packet capture and transmission. It is under    *
- * separate license terms which forbid redistribution without special      *
- * permission. So the official Nmap Windows builds may not be              *
- * redistributed without special permission (such as an Nmap OEM           *
- * license).                                                               *
- *                                                                         *
- * Source is provided to this software because we believe users have a     *
- * right to know exactly what a program is going to do before they run it. *
- * This also allows you to audit the software for security holes.          *
- *                                                                         *
- * Source code also allows you to port Nmap to new platforms, fix bugs,    *
- * and add new features.  You are highly encouraged to submit your         *
- * changes as a Github PR or by email to the dev@nmap.org mailing list     *
- * for possible incorporation into the main distribution. Unless you       *
- * specify otherwise, it is understood that you are offering us very       *
- * broad rights to use your submissions as described in the Nmap Public    *
- * Source License Contributor Agreement. This is important because we      *
- * fund the project by selling licenses with various terms, and also       *
- * because the inability to relicense code has caused devastating          *
- * problems for other Free Software projects (such as KDE and NASM).       *
- *                                                                         *
- * The free version of Nmap is distributed in the hope that it will be     *
- * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of  *
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. Warranties,        *
- * indemnification and commercial support are all available through the    *
- * Npcap OEM program--see https://nmap.org/oem.                            *
- *                                                                         *
+ *
+ * The Nmap Security Scanner is (C) 1996-2023 Nmap Software LLC ("The Nmap
+ * Project"). Nmap is also a registered trademark of the Nmap Project.
+ *
+ * This program is distributed under the terms of the Nmap Public Source
+ * License (NPSL). The exact license text applying to a particular Nmap
+ * release or source code control revision is contained in the LICENSE
+ * file distributed with that version of Nmap or source code control
+ * revision. More Nmap copyright/legal information is available from
+ * https://nmap.org/book/man-legal.html, and further information on the
+ * NPSL license itself can be found at https://nmap.org/npsl/ . This
+ * header summarizes some key points from the Nmap license, but is no
+ * substitute for the actual license text.
+ *
+ * Nmap is generally free for end users to download and use themselves,
+ * including commercial use. It is available from https://nmap.org.
+ *
+ * The Nmap license generally prohibits companies from using and
+ * redistributing Nmap in commercial products, but we sell a special Nmap
+ * OEM Edition with a more permissive license and special features for
+ * this purpose. See https://nmap.org/oem/
+ *
+ * If you have received a written Nmap license agreement or contract
+ * stating terms other than these (such as an Nmap OEM license), you may
+ * choose to use and redistribute Nmap under those terms instead.
+ *
+ * The official Nmap Windows builds include the Npcap software
+ * (https://npcap.com) for packet capture and transmission. It is under
+ * separate license terms which forbid redistribution without special
+ * permission. So the official Nmap Windows builds may not be redistributed
+ * without special permission (such as an Nmap OEM license).
+ *
+ * Source is provided to this software because we believe users have a
+ * right to know exactly what a program is going to do before they run it.
+ * This also allows you to audit the software for security holes.
+ *
+ * Source code also allows you to port Nmap to new platforms, fix bugs, and add
+ * new features. You are highly encouraged to submit your changes as a Github PR
+ * or by email to the dev@nmap.org mailing list for possible incorporation into
+ * the main distribution. Unless you specify otherwise, it is understood that
+ * you are offering us very broad rights to use your submissions as described in
+ * the Nmap Public Source License Contributor Agreement. This is important
+ * because we fund the project by selling licenses with various terms, and also
+ * because the inability to relicense code has caused devastating problems for
+ * other Free Software projects (such as KDE and NASM).
+ *
+ * The free version of Nmap is distributed in the hope that it will be
+ * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. Warranties,
+ * indemnification and commercial support are all available through the
+ * Npcap OEM program--see https://nmap.org/oem/
+ *
  ***************************************************************************/
 
 /* Convert a string like "-100,n*tp,200-1024,3000-4000,[60000-]" into an array
@@ -337,39 +335,26 @@ static void getpts_aux(const char *origexpr, int nested, u8 *porttbl, int range_
     while (isspace((int) (unsigned char) *current_range))
       current_range++; /* I don't know why I should allow spaces here, but I will */
 
-    if (change_range_type) {
-      if (*current_range == 'T' && *(current_range+1) == ':') {
-        current_range += 2;
-        range_type = SCAN_TCP_PORT;
-        if (!o.TCPScan()) {
-          error("WARNING: Your ports include \"T:\" but you haven't specified any TCP scan type.");
-        }
-        continue;
-      }
-      if (*current_range == 'U' && *(current_range+1) == ':') {
-        current_range += 2;
-        range_type = SCAN_UDP_PORT;
-        if (!o.UDPScan()) {
-          error("WARNING: Your ports include \"U:\" but you haven't specified UDP scan with -sU.");
-        }
-        continue;
-      }
-      if (*current_range == 'S' && *(current_range+1) == ':') {
-        current_range += 2;
-        range_type = SCAN_SCTP_PORT;
-        if (!o.SCTPScan()) {
-          error("WARNING: Your ports include \"S:\" but you haven't specified any SCTP scan type.");
-        }
-        continue;
-      }
-      if (*current_range == 'P' && *(current_range+1) == ':') {
-        current_range += 2;
-        range_type = SCAN_PROTOCOLS;
-        if (!o.ipprotscan) {
-          error("WARNING: Your ports include \"P:\" but you haven't specified IP Protocol scan with -sO.");
-        }
-        continue;
+    if (change_range_type && *(current_range+1) == ':') {
+      switch (*current_range) {
+        case 'T':
+          range_type = SCAN_TCP_PORT;
+          break;
+        case 'U':
+          range_type = SCAN_UDP_PORT;
+          break;
+        case 'S':
+          range_type = SCAN_SCTP_PORT;
+          break;
+        case 'P':
+          range_type = SCAN_PROTOCOLS;
+          break;
+        default:
+          fatal("Error parsing port list: Unknown protocol specifier '%c'.", *current_range);
+          break;
       }
+      current_range += 2;
+      continue;
     }
 
     if (*current_range == '[') {
@@ -478,15 +463,15 @@ static void getpts_aux(const char *origexpr, int nested, u8 *porttbl, int range_
       } else {
         if (nested) {
           if ((range_type & SCAN_TCP_PORT) &&
-              nmap_getservbyport(rangestart, "tcp")) {
+              nmap_getservbyport(rangestart, IPPROTO_TCP)) {
             porttbl[rangestart] |= SCAN_TCP_PORT;
           }
           if ((range_type & SCAN_UDP_PORT) &&
-              nmap_getservbyport(rangestart, "udp")) {
+              nmap_getservbyport(rangestart, IPPROTO_UDP)) {
             porttbl[rangestart] |= SCAN_UDP_PORT;
           }
           if ((range_type & SCAN_SCTP_PORT) &&
-              nmap_getservbyport(rangestart, "sctp")) {
+              nmap_getservbyport(rangestart, IPPROTO_SCTP)) {
             porttbl[rangestart] |= SCAN_SCTP_PORT;
           }
           if ((range_type & SCAN_PROTOCOLS) &&
