@@ -5,139 +5,69 @@
  * formats.                                                                *
  *                                                                         *
  ***********************IMPORTANT NMAP LICENSE TERMS************************
- *                                                                         *
- * The Nmap Security Scanner is (C) 1996-2017 Insecure.Com LLC ("The Nmap  *
- * Project"). Nmap is also a registered trademark of the Nmap Project.     *
- * This program is free software; you may redistribute and/or modify it    *
- * under the terms of the GNU General Public License as published by the   *
- * Free Software Foundation; Version 2 ("GPL"), BUT ONLY WITH ALL OF THE   *
- * CLARIFICATIONS AND EXCEPTIONS DESCRIBED HEREIN.  This guarantees your   *
- * right to use, modify, and redistribute this software under certain      *
- * conditions.  If you wish to embed Nmap technology into proprietary      *
- * software, we sell alternative licenses (contact sales@nmap.com).        *
- * Dozens of software vendors already license Nmap technology such as      *
- * host discovery, port scanning, OS detection, version detection, and     *
- * the Nmap Scripting Engine.                                              *
- *                                                                         *
- * Note that the GPL places important restrictions on "derivative works",  *
- * yet it does not provide a detailed definition of that term.  To avoid   *
- * misunderstandings, we interpret that term as broadly as copyright law   *
- * allows.  For example, we consider an application to constitute a        *
- * derivative work for the purpose of this license if it does any of the   *
- * following with any software or content covered by this license          *
- * ("Covered Software"):                                                   *
- *                                                                         *
- * o Integrates source code from Covered Software.                         *
- *                                                                         *
- * o Reads or includes copyrighted data files, such as Nmap's nmap-os-db   *
- * or nmap-service-probes.                                                 *
- *                                                                         *
- * o Is designed specifically to execute Covered Software and parse the    *
- * results (as opposed to typical shell or execution-menu apps, which will *
- * execute anything you tell them to).                                     *
- *                                                                         *
- * o Includes Covered Software in a proprietary executable installer.  The *
- * installers produced by InstallShield are an example of this.  Including *
- * Nmap with other software in compressed or archival form does not        *
- * trigger this provision, provided appropriate open source decompression  *
- * or de-archiving software is widely available for no charge.  For the    *
- * purposes of this license, an installer is considered to include Covered *
- * Software even if it actually retrieves a copy of Covered Software from  *
- * another source during runtime (such as by downloading it from the       *
- * Internet).                                                              *
- *                                                                         *
- * o Links (statically or dynamically) to a library which does any of the  *
- * above.                                                                  *
- *                                                                         *
- * o Executes a helper program, module, or script to do any of the above.  *
- *                                                                         *
- * This list is not exclusive, but is meant to clarify our interpretation  *
- * of derived works with some common examples.  Other people may interpret *
- * the plain GPL differently, so we consider this a special exception to   *
- * the GPL that we apply to Covered Software.  Works which meet any of     *
- * these conditions must conform to all of the terms of this license,      *
- * particularly including the GPL Section 3 requirements of providing      *
- * source code and allowing free redistribution of the work as a whole.    *
- *                                                                         *
- * As another special exception to the GPL terms, the Nmap Project grants  *
- * permission to link the code of this program with any version of the     *
- * OpenSSL library which is distributed under a license identical to that  *
- * listed in the included docs/licenses/OpenSSL.txt file, and distribute   *
- * linked combinations including the two.                                  *
- *                                                                         *
- * The Nmap Project has permission to redistribute Npcap, a packet         *
- * capturing driver and library for the Microsoft Windows platform.        *
- * Npcap is a separate work with it's own license rather than this Nmap    *
- * license.  Since the Npcap license does not permit redistribution        *
- * without special permission, our Nmap Windows binary packages which      *
- * contain Npcap may not be redistributed without special permission.      *
- *                                                                         *
- * Any redistribution of Covered Software, including any derived works,    *
- * must obey and carry forward all of the terms of this license, including *
- * obeying all GPL rules and restrictions.  For example, source code of    *
- * the whole work must be provided and free redistribution must be         *
- * allowed.  All GPL references to "this License", are to be treated as    *
- * including the terms and conditions of this license text as well.        *
- *                                                                         *
- * Because this license imposes special exceptions to the GPL, Covered     *
- * Work may not be combined (even as part of a larger work) with plain GPL *
- * software.  The terms, conditions, and exceptions of this license must   *
- * be included as well.  This license is incompatible with some other open *
- * source licenses as well.  In some cases we can relicense portions of    *
- * Nmap or grant special permissions to use it in other open source        *
- * software.  Please contact fyodor@nmap.org with any such requests.       *
- * Similarly, we don't incorporate incompatible open source software into  *
- * Covered Software without special permission from the copyright holders. *
- *                                                                         *
- * If you have any questions about the licensing restrictions on using     *
- * Nmap in other works, are happy to help.  As mentioned above, we also    *
- * offer alternative license to integrate Nmap into proprietary            *
- * applications and appliances.  These contracts have been sold to dozens  *
- * of software vendors, and generally include a perpetual license as well  *
- * as providing for priority support and updates.  They also fund the      *
- * continued development of Nmap.  Please email sales@nmap.com for further *
- * information.                                                            *
- *                                                                         *
- * If you have received a written license agreement or contract for        *
- * Covered Software stating terms other than these, you may choose to use  *
- * and redistribute Covered Software under those terms instead of these.   *
- *                                                                         *
- * Source is provided to this software because we believe users have a     *
- * right to know exactly what a program is going to do before they run it. *
- * This also allows you to audit the software for security holes.          *
- *                                                                         *
- * Source code also allows you to port Nmap to new platforms, fix bugs,    *
- * and add new features.  You are highly encouraged to send your changes   *
- * to the dev@nmap.org mailing list for possible incorporation into the    *
- * main distribution.  By sending these changes to Fyodor or one of the    *
- * Insecure.Org development mailing lists, or checking them into the Nmap  *
- * source code repository, it is understood (unless you specify            *
- * otherwise) that you are offering the Nmap Project the unlimited,        *
- * non-exclusive right to reuse, modify, and relicense the code.  Nmap     *
- * will always be available Open Source, but this is important because     *
- * the inability to relicense code has caused devastating problems for     *
- * other Free Software projects (such as KDE and NASM).  We also           *
- * occasionally relicense the code to third parties as discussed above.    *
- * If you wish to specify special license conditions of your               *
- * contributions, just say so when you send them.                          *
- *                                                                         *
- * This program is distributed in the hope that it will be useful, but     *
- * WITHOUT ANY WARRANTY; without even the implied warranty of              *
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the Nmap      *
- * license file for more details (it's in a COPYING file included with     *
- * Nmap, and also available from https://svn.nmap.org/nmap/COPYING)        *
- *                                                                         *
+ *
+ * The Nmap Security Scanner is (C) 1996-2023 Nmap Software LLC ("The Nmap
+ * Project"). Nmap is also a registered trademark of the Nmap Project.
+ *
+ * This program is distributed under the terms of the Nmap Public Source
+ * License (NPSL). The exact license text applying to a particular Nmap
+ * release or source code control revision is contained in the LICENSE
+ * file distributed with that version of Nmap or source code control
+ * revision. More Nmap copyright/legal information is available from
+ * https://nmap.org/book/man-legal.html, and further information on the
+ * NPSL license itself can be found at https://nmap.org/npsl/ . This
+ * header summarizes some key points from the Nmap license, but is no
+ * substitute for the actual license text.
+ *
+ * Nmap is generally free for end users to download and use themselves,
+ * including commercial use. It is available from https://nmap.org.
+ *
+ * The Nmap license generally prohibits companies from using and
+ * redistributing Nmap in commercial products, but we sell a special Nmap
+ * OEM Edition with a more permissive license and special features for
+ * this purpose. See https://nmap.org/oem/
+ *
+ * If you have received a written Nmap license agreement or contract
+ * stating terms other than these (such as an Nmap OEM license), you may
+ * choose to use and redistribute Nmap under those terms instead.
+ *
+ * The official Nmap Windows builds include the Npcap software
+ * (https://npcap.com) for packet capture and transmission. It is under
+ * separate license terms which forbid redistribution without special
+ * permission. So the official Nmap Windows builds may not be redistributed
+ * without special permission (such as an Nmap OEM license).
+ *
+ * Source is provided to this software because we believe users have a
+ * right to know exactly what a program is going to do before they run it.
+ * This also allows you to audit the software for security holes.
+ *
+ * Source code also allows you to port Nmap to new platforms, fix bugs, and add
+ * new features. You are highly encouraged to submit your changes as a Github PR
+ * or by email to the dev@nmap.org mailing list for possible incorporation into
+ * the main distribution. Unless you specify otherwise, it is understood that
+ * you are offering us very broad rights to use your submissions as described in
+ * the Nmap Public Source License Contributor Agreement. This is important
+ * because we fund the project by selling licenses with various terms, and also
+ * because the inability to relicense code has caused devastating problems for
+ * other Free Software projects (such as KDE and NASM).
+ *
+ * The free version of Nmap is distributed in the hope that it will be
+ * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. Warranties,
+ * indemnification and commercial support are all available through the
+ * Npcap OEM program--see https://nmap.org/oem/
+ *
  ***************************************************************************/
 
-/* $Id: targets.cc 36788 2017-06-07 12:32:38Z dmiller $ */
+/* $Id: targets.cc 38653 2023-04-14 17:11:46Z dmiller $ */
 
 
-#include "nbase/nbase_addrset.h"
+#include <nbase.h>
 #include "targets.h"
 #include "timing.h"
 #include "tcpip.h"
 #include "NmapOps.h"
-#include "TargetGroup.h"
+#include "NewTargets.h"
 #include "Target.h"
 #include "scan_engine.h"
 #include "nmap_dns.h"
@@ -206,7 +136,7 @@ void returnhost(HostGroupState *hs) {
 /* Is the host passed as Target to be excluded? Much of this logic had
    to be rewritten from wam's original code to allow for the objects */
 static int hostInExclude(struct sockaddr *checksock, size_t checksocklen,
-                  const addrset *exclude_group) {
+                  const struct addrset *exclude_group) {
   if (exclude_group == NULL)
     return 0;
 
@@ -219,7 +149,7 @@ static int hostInExclude(struct sockaddr *checksock, size_t checksocklen,
 }
 
 /* Load an exclude list from a file for --excludefile. */
-int load_exclude_file(addrset *excludelist, FILE *fp) {
+int load_exclude_file(struct addrset *excludelist, FILE *fp) {
   char host_spec[1024];
   size_t n;
 
@@ -236,7 +166,7 @@ int load_exclude_file(addrset *excludelist, FILE *fp) {
 
 /* Load a comma-separated exclude list from a string, the argument to
    --exclude. */
-int load_exclude_string(addrset *excludelist, const char *s) {
+int load_exclude_string(struct addrset *excludelist, const char *s) {
   const char *begin, *p;
 
   p = s;
@@ -259,16 +189,12 @@ int load_exclude_string(addrset *excludelist, const char *s) {
 
 /* A debug routine to dump some information to stdout. Invoked if debugging is
    set to 4 or higher. */
-int dumpExclude(addrset *exclude_group) {
-  const struct addrset_elem *elem;
-
-  for (elem = exclude_group->head; elem != NULL; elem = elem->next)
-    addrset_elem_print(stdout, elem);
-
+int dumpExclude(const struct addrset *exclude_group) {
+  addrset_print(stdout, exclude_group);
   return 1;
 }
 
-static void massping(Target *hostbatch[], int num_hosts, struct scan_lists *ports) {
+static void massping(Target *hostbatch[], int num_hosts, const struct scan_lists *ports) {
   static struct timeout_info group_to = { 0, 0, 0 };
   static char prev_device_name[16] = "";
   const char *device_name;
@@ -294,6 +220,8 @@ static void massping(Target *hostbatch[], int num_hosts, struct scan_lists *port
   }
 
   for (i = 0; i < num_hosts; i++) {
+    if (hostbatch[i]->flags & HOST_DOWN)
+      continue;
     initialize_timeout_info(&hostbatch[i]->to);
     targets.push_back(hostbatch[i]);
   }
@@ -307,9 +235,7 @@ static void massping(Target *hostbatch[], int num_hosts, struct scan_lists *port
      2. it uses a different source address, or
      3. it is directly connected when the other hosts are not, or vice versa, or
      4. it has the same IP address as another target already in the group.
-   These restrictions only apply for raw scans. This function is similar to one
-   of the same name in nmap.cc. That one is for port scanning, this one is for
-   ping scanning. */
+   These restrictions only apply for raw scans, including host discovery. */
 bool target_needs_new_hostgroup(Target **targets, int targets_sz, const Target *target) {
   int i = 0;
 
@@ -352,95 +278,6 @@ bool target_needs_new_hostgroup(Target **targets, int targets_sz, const Target *
   return false;
 }
 
-TargetGroup::~TargetGroup() {
-  if (this->netblock != NULL)
-    delete this->netblock;
-}
-
-/* Initializes (or reinitializes) the object with a new expression, such
-   as 192.168.0.0/16 , 10.1.0-5.1-254 , or fe80::202:e3ff:fe14:1102 .
-   Returns 0 for success */
-int TargetGroup::parse_expr(const char *target_expr, int af) {
-  if (this->netblock != NULL)
-    delete this->netblock;
-  this->netblock = NetBlock::parse_expr(target_expr, af);
-  if (this->netblock != NULL)
-    return 0;
-  else
-    return 1;
-}
-
-/* Grab the next host from this expression (if any) and updates its internal
-   state to reflect that the IP was given out.  Returns 0 and
-   fills in ss if successful.  ss must point to a pre-allocated
-   sockaddr_storage structure */
-int TargetGroup::get_next_host(struct sockaddr_storage *ss, size_t *sslen) {
-  if (this->netblock == NULL)
-    return -1;
-
-  /* If all we have at this point is a hostname and netmask, resolve into
-     something where we know the address. If we ever have to use strictly the
-     hostname, without doing local DNS resolution (like with a proxy scan), this
-     has to be made conditional (and perhaps an error if the netmask doesn't
-     limit it to exactly one address). */
-  NetBlockHostname *netblock_hostname;
-  netblock_hostname = dynamic_cast<NetBlockHostname *>(this->netblock);
-  if (netblock_hostname != NULL) {
-    this->netblock = netblock_hostname->resolve();
-    if (this->netblock == NULL) {
-      error("Failed to resolve \"%s\".", netblock_hostname->hostname.c_str());
-      delete netblock_hostname;
-      return -1;
-    }
-    delete netblock_hostname;
-  }
-
-  /* Check for proper address family. Give a specific error message for IPv6
-     specifications appearing in IPv4 mode. */
-  if (o.af() == AF_INET && dynamic_cast<NetBlockIPv6Netmask *>(this->netblock) != NULL) {
-    error("%s looks like an IPv6 target specification -- you have to use the -6 option.",
-      this->netblock->str().c_str());
-    return -1;
-  }
-  if ((o.af() == AF_INET && dynamic_cast<NetBlockIPv4Ranges *>(this->netblock) == NULL) ||
-      (o.af() == AF_INET6 && dynamic_cast<NetBlockIPv6Netmask *>(this->netblock) == NULL)) {
-    error("Address family mismatch in target specification \"%s\".",
-      this->netblock->str().c_str());
-    return -1;
-  }
-
-  if (this->netblock->next(ss, sslen))
-    return 0;
-  else
-    return -1;
-}
-
-/* Returns true iff the given address is the one that was resolved to create
-   this target group; i.e., not one of the addresses derived from it with a
-   netmask. */
-bool TargetGroup::is_resolved_address(const struct sockaddr_storage *ss) const {
-  return this->netblock->is_resolved_address(ss);
-}
-
-/* Return a string of the name or address that was resolved for this group. */
-const char *TargetGroup::get_resolved_name(void) const {
-  if (this->netblock->hostname.empty())
-    return NULL;
-  else
-    return this->netblock->hostname.c_str();
-}
-
-/* Return the list of addresses that the name for this group resolved to, if
-   it came from a name resolution. */
-const std::list<struct sockaddr_storage> &TargetGroup::get_resolved_addrs(void) const {
-  return this->netblock->resolvedaddrs;
-}
-
-/* is the current expression a named host */
-int TargetGroup::get_namedhost() const {
-  return this->get_resolved_name() != NULL;
-}
-
 /* Lookahead is the number of hosts that can be
    checked (such as ping scanned) in advance.  Randomize causes each
    group of up to lookahead hosts to be internally shuffled around.
@@ -486,15 +323,15 @@ const char *HostGroupState::next_expression() {
   /* Add any new NSE discovered targets to the scan queue */
   static char buf[1024];
 
-  NewTargets *new_targets = NewTargets::get();
-  if (o.script && new_targets != NULL) {
-    if (new_targets->get_queued() > 0) {
+  if (o.script) {
+    unsigned long new_targets = NewTargets::get_queued();
+    if (new_targets > 0) {
       std::string expr_string;
-      expr_string = new_targets->read().c_str();
+      expr_string = NewTargets::read().c_str();
       if (o.debugging > 3) {
         log_write(LOG_PLAIN,
-                  "New targets in the scanned cache: %ld, pending ones: %ld.\n",
-                  new_targets->get_scanned(), new_targets->get_queued());
+                  "New targets: retrieved one of %ld pending in queue.\n",
+                  new_targets);
       }
       if (!expr_string.empty()) {
         Strncpy(buf, expr_string.c_str(), sizeof(buf));
@@ -536,20 +373,12 @@ static Target *setup_target(const HostGroupState *hs,
   if (hs->current_group.is_resolved_address(ss)) {
     if (hs->current_group.get_namedhost())
       t->setTargetName(hs->current_group.get_resolved_name());
-    t->resolved_addrs = hs->current_group.get_resolved_addrs();
+    t->unscanned_addrs = hs->current_group.get_unscanned_addrs();
   }
 
   /* We figure out the source IP/device IFF
-     1) We are r00t AND
-     2) We are doing tcp or udp pingscan OR
-     3) We are doing a raw-mode portscan or osscan or traceroute OR
-     4) We are on windows and doing ICMP ping */
-  if (o.isr00t &&
-      ((pingtype & (PINGTYPE_TCP|PINGTYPE_UDP|PINGTYPE_SCTP_INIT|PINGTYPE_PROTO|PINGTYPE_ARP)) || o.RawScan()
-#ifdef WIN32
-       || (pingtype & (PINGTYPE_ICMP_PING|PINGTYPE_ICMP_MASK|PINGTYPE_ICMP_TS))
-#endif // WIN32
-      )) {
+   * the scan type requires us to */
+  if (o.RawScan()) {
     if (!nmap_route_dst(ss, &rnfo)) {
       log_bogus_target(inet_ntop_ez(ss, sslen));
       error("%s: failed to determine route to %s", __func__, t->NameIP());
@@ -592,8 +421,8 @@ bail:
   return NULL;
 }
 
-static Target *next_target(HostGroupState *hs, const addrset *exclude_group,
-  struct scan_lists *ports, int pingtype) {
+static Target *next_target(HostGroupState *hs, struct addrset *exclude_group,
+  const struct scan_lists *ports, int pingtype) {
   struct sockaddr_storage ss;
   size_t sslen;
   Target *t;
@@ -627,10 +456,10 @@ tryagain:
 
   /* If we are resuming from a previous scan, we have already finished scanning
      up to o.resume_ip.  */
-  if (ss.ss_family == AF_INET && o.resume_ip.s_addr) {
-    if (o.resume_ip.s_addr == ((struct sockaddr_in *) &ss)->sin_addr.s_addr)
+  if (o.resume_ip.ss_family != AF_UNSPEC) {
+    if (!sockaddr_storage_cmp(&o.resume_ip, &ss))
       /* We will continue starting with the next IP. */
-      o.resume_ip.s_addr = 0;
+      o.resume_ip.ss_family = AF_UNSPEC;
     goto tryagain;
   }
 
@@ -642,11 +471,15 @@ tryagain:
   if (t == NULL)
     goto tryagain;
 
+  if (o.unique) {
+    // Use the exclude list to avoid scanning this IP again if the user requested it.
+    addrset_add_spec(exclude_group, t->targetipstr(), o.af(), 0);
+  }
   return t;
 }
 
-static void refresh_hostbatch(HostGroupState *hs, const addrset *exclude_group,
-  struct scan_lists *ports, int pingtype) {
+static void refresh_hostbatch(HostGroupState *hs, struct addrset *exclude_group,
+  const struct scan_lists *ports, int pingtype) {
   int i;
   bool arpping_done = false;
   struct timeval now;
@@ -688,7 +521,7 @@ static void refresh_hostbatch(HostGroupState *hs, const addrset *exclude_group,
       hs->hostbatch[0]->af() == AF_INET &&
       hs->hostbatch[0]->directlyConnected() &&
       o.sendpref != PACKET_SEND_IP_STRONG &&
-      (pingtype == PINGTYPE_ARP || o.implicitARPPing)) {
+      o.implicitARPPing) {
     arpping(hs->hostbatch, hs->current_batch_sz);
     arpping_done = true;
   }
@@ -699,7 +532,7 @@ static void refresh_hostbatch(HostGroupState *hs, const addrset *exclude_group,
       hs->hostbatch[0]->af() == AF_INET6 &&
       hs->hostbatch[0]->directlyConnected() &&
       o.sendpref != PACKET_SEND_IP_STRONG &&
-      (pingtype == PINGTYPE_ARP || o.implicitARPPing)) {
+      o.implicitARPPing) {
     arpping(hs->hostbatch, hs->current_batch_sz);
     arpping_done = true;
   }
@@ -711,8 +544,9 @@ static void refresh_hostbatch(HostGroupState *hs, const addrset *exclude_group,
       if (!(hs->hostbatch[i]->flags & HOST_DOWN) &&
           !hs->hostbatch[i]->timedOut(&now)) {
         if (!setTargetNextHopMAC(hs->hostbatch[i])) {
-          fatal("%s: Failed to determine dst MAC address for target %s",
+          error("%s: Failed to determine dst MAC address for target %s",
               __func__, hs->hostbatch[i]->NameIP());
+          hs->hostbatch[i]->flags = HOST_DOWN;
         }
       }
     }
@@ -721,7 +555,7 @@ static void refresh_hostbatch(HostGroupState *hs, const addrset *exclude_group,
   /* Then we do the mass ping (if required - IP-level pings) */
   if ((pingtype == PINGTYPE_NONE && !arpping_done) || hs->hostbatch[0]->ifType() == devt_loopback) {
     for (i=0; i < hs->current_batch_sz; i++) {
-      if (!hs->hostbatch[i]->timedOut(&now)) {
+      if (!(hs->hostbatch[i]->flags & HOST_DOWN || hs->hostbatch[i]->timedOut(&now))) {
         initialize_timeout_info(&hs->hostbatch[i]->to);
         hs->hostbatch[i]->flags |= HOST_UP; /*hostbatch[i].up = 1;*/
         if (pingtype == PINGTYPE_NONE && !arpping_done)
@@ -738,8 +572,8 @@ static void refresh_hostbatch(HostGroupState *hs, const addrset *exclude_group,
     nmap_mass_rdns(hs->hostbatch, hs->current_batch_sz);
 }
 
-Target *nexthost(HostGroupState *hs, const addrset *exclude_group,
-                 struct scan_lists *ports, int pingtype) {
+Target *nexthost(HostGroupState *hs, struct addrset *exclude_group,
+                 const struct scan_lists *ports, int pingtype) {
   if (hs->next_batch_no >= hs->current_batch_sz)
     refresh_hostbatch(hs, exclude_group, ports, pingtype);
   if (hs->next_batch_no >= hs->current_batch_sz)
