@@ -1,126 +1,64 @@
-#!/usr/bin/env python
-# -*- coding: utf-8 -*-
+#!/usr/bin/env python3
 
 # ***********************IMPORTANT NMAP LICENSE TERMS************************
-# *                                                                         *
-# * The Nmap Security Scanner is (C) 1996-2016 Insecure.Com LLC. Nmap is    *
-# * also a registered trademark of Insecure.Com LLC.  This program is free  *
-# * software; you may redistribute and/or modify it under the terms of the  *
-# * GNU General Public License as published by the Free Software            *
-# * Foundation; Version 2 ("GPL"), BUT ONLY WITH ALL OF THE CLARIFICATIONS  *
-# * AND EXCEPTIONS DESCRIBED HEREIN.  This guarantees your right to use,    *
-# * modify, and redistribute this software under certain conditions.  If    *
-# * you wish to embed Nmap technology into proprietary software, we sell    *
-# * alternative licenses (contact sales@nmap.com).  Dozens of software      *
-# * vendors already license Nmap technology such as host discovery, port    *
-# * scanning, OS detection, version detection, and the Nmap Scripting       *
-# * Engine.                                                                 *
-# *                                                                         *
-# * Note that the GPL places important restrictions on "derivative works",  *
-# * yet it does not provide a detailed definition of that term.  To avoid   *
-# * misunderstandings, we interpret that term as broadly as copyright law   *
-# * allows.  For example, we consider an application to constitute a        *
-# * derivative work for the purpose of this license if it does any of the   *
-# * following with any software or content covered by this license          *
-# * ("Covered Software"):                                                   *
-# *                                                                         *
-# * o Integrates source code from Covered Software.                         *
-# *                                                                         *
-# * o Reads or includes copyrighted data files, such as Nmap's nmap-os-db   *
-# * or nmap-service-probes.                                                 *
-# *                                                                         *
-# * o Is designed specifically to execute Covered Software and parse the    *
-# * results (as opposed to typical shell or execution-menu apps, which will *
-# * execute anything you tell them to).                                     *
-# *                                                                         *
-# * o Includes Covered Software in a proprietary executable installer.  The *
-# * installers produced by InstallShield are an example of this.  Including *
-# * Nmap with other software in compressed or archival form does not        *
-# * trigger this provision, provided appropriate open source decompression  *
-# * or de-archiving software is widely available for no charge.  For the    *
-# * purposes of this license, an installer is considered to include Covered *
-# * Software even if it actually retrieves a copy of Covered Software from  *
-# * another source during runtime (such as by downloading it from the       *
-# * Internet).                                                              *
-# *                                                                         *
-# * o Links (statically or dynamically) to a library which does any of the  *
-# * above.                                                                  *
-# *                                                                         *
-# * o Executes a helper program, module, or script to do any of the above.  *
-# *                                                                         *
-# * This list is not exclusive, but is meant to clarify our interpretation  *
-# * of derived works with some common examples.  Other people may interpret *
-# * the plain GPL differently, so we consider this a special exception to   *
-# * the GPL that we apply to Covered Software.  Works which meet any of     *
-# * these conditions must conform to all of the terms of this license,      *
-# * particularly including the GPL Section 3 requirements of providing      *
-# * source code and allowing free redistribution of the work as a whole.    *
-# *                                                                         *
-# * As another special exception to the GPL terms, Insecure.Com LLC grants  *
-# * permission to link the code of this program with any version of the     *
-# * OpenSSL library which is distributed under a license identical to that  *
-# * listed in the included docs/licenses/OpenSSL.txt file, and distribute   *
-# * linked combinations including the two.                                  *
-# *                                                                         *
-# * Any redistribution of Covered Software, including any derived works,    *
-# * must obey and carry forward all of the terms of this license, including *
-# * obeying all GPL rules and restrictions.  For example, source code of    *
-# * the whole work must be provided and free redistribution must be         *
-# * allowed.  All GPL references to "this License", are to be treated as    *
-# * including the terms and conditions of this license text as well.        *
-# *                                                                         *
-# * Because this license imposes special exceptions to the GPL, Covered     *
-# * Work may not be combined (even as part of a larger work) with plain GPL *
-# * software.  The terms, conditions, and exceptions of this license must   *
-# * be included as well.  This license is incompatible with some other open *
-# * source licenses as well.  In some cases we can relicense portions of    *
-# * Nmap or grant special permissions to use it in other open source        *
-# * software.  Please contact fyodor@nmap.org with any such requests.       *
-# * Similarly, we don't incorporate incompatible open source software into  *
-# * Covered Software without special permission from the copyright holders. *
-# *                                                                         *
-# * If you have any questions about the licensing restrictions on using     *
-# * Nmap in other works, are happy to help.  As mentioned above, we also    *
-# * offer alternative license to integrate Nmap into proprietary            *
-# * applications and appliances.  These contracts have been sold to dozens  *
-# * of software vendors, and generally include a perpetual license as well  *
-# * as providing for priority support and updates.  They also fund the      *
-# * continued development of Nmap.  Please email sales@nmap.com for further *
-# * information.                                                            *
-# *                                                                         *
-# * If you have received a written license agreement or contract for        *
-# * Covered Software stating terms other than these, you may choose to use  *
-# * and redistribute Covered Software under those terms instead of these.   *
-# *                                                                         *
-# * Source is provided to this software because we believe users have a     *
-# * right to know exactly what a program is going to do before they run it. *
-# * This also allows you to audit the software for security holes.          *
-# *                                                                         *
-# * Source code also allows you to port Nmap to new platforms, fix bugs,    *
-# * and add new features.  You are highly encouraged to send your changes   *
-# * to the dev@nmap.org mailing list for possible incorporation into the    *
-# * main distribution.  By sending these changes to Fyodor or one of the    *
-# * Insecure.Org development mailing lists, or checking them into the Nmap  *
-# * source code repository, it is understood (unless you specify otherwise) *
-# * that you are offering the Nmap Project (Insecure.Com LLC) the           *
-# * unlimited, non-exclusive right to reuse, modify, and relicense the      *
-# * code.  Nmap will always be available Open Source, but this is important *
-# * because the inability to relicense code has caused devastating problems *
-# * for other Free Software projects (such as KDE and NASM).  We also       *
-# * occasionally relicense the code to third parties as discussed above.    *
-# * If you wish to specify special license conditions of your               *
-# * contributions, just say so when you send them.                          *
-# *                                                                         *
-# * This program is distributed in the hope that it will be useful, but     *
-# * WITHOUT ANY WARRANTY; without even the implied warranty of              *
-# * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the Nmap      *
-# * license file for more details (it's in a COPYING file included with     *
-# * Nmap, and also available from https://svn.nmap.org/nmap/COPYING)        *
-# *                                                                         *
+# *
+# * The Nmap Security Scanner is (C) 1996-2023 Nmap Software LLC ("The Nmap
+# * Project"). Nmap is also a registered trademark of the Nmap Project.
+# *
+# * This program is distributed under the terms of the Nmap Public Source
+# * License (NPSL). The exact license text applying to a particular Nmap
+# * release or source code control revision is contained in the LICENSE
+# * file distributed with that version of Nmap or source code control
+# * revision. More Nmap copyright/legal information is available from
+# * https://nmap.org/book/man-legal.html, and further information on the
+# * NPSL license itself can be found at https://nmap.org/npsl/ . This
+# * header summarizes some key points from the Nmap license, but is no
+# * substitute for the actual license text.
+# *
+# * Nmap is generally free for end users to download and use themselves,
+# * including commercial use. It is available from https://nmap.org.
+# *
+# * The Nmap license generally prohibits companies from using and
+# * redistributing Nmap in commercial products, but we sell a special Nmap
+# * OEM Edition with a more permissive license and special features for
+# * this purpose. See https://nmap.org/oem/
+# *
+# * If you have received a written Nmap license agreement or contract
+# * stating terms other than these (such as an Nmap OEM license), you may
+# * choose to use and redistribute Nmap under those terms instead.
+# *
+# * The official Nmap Windows builds include the Npcap software
+# * (https://npcap.com) for packet capture and transmission. It is under
+# * separate license terms which forbid redistribution without special
+# * permission. So the official Nmap Windows builds may not be redistributed
+# * without special permission (such as an Nmap OEM license).
+# *
+# * Source is provided to this software because we believe users have a
+# * right to know exactly what a program is going to do before they run it.
+# * This also allows you to audit the software for security holes.
+# *
+# * Source code also allows you to port Nmap to new platforms, fix bugs, and add
+# * new features. You are highly encouraged to submit your changes as a Github PR
+# * or by email to the dev@nmap.org mailing list for possible incorporation into
+# * the main distribution. Unless you specify otherwise, it is understood that
+# * you are offering us very broad rights to use your submissions as described in
+# * the Nmap Public Source License Contributor Agreement. This is important
+# * because we fund the project by selling licenses with various terms, and also
+# * because the inability to relicense code has caused devastating problems for
+# * other Free Software projects (such as KDE and NASM).
+# *
+# * The free version of Nmap is distributed in the hope that it will be
+# * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
+# * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. Warranties,
+# * indemnification and commercial support are all available through the
+# * Npcap OEM program--see https://nmap.org/oem/
+# *
 # ***************************************************************************/
 
-import gobject
-import gtk
+import gi
+
+gi.require_version("Gtk", "3.0")
+from gi.repository import Gtk, GObject
 
 
 # Prevent loading PyXML
@@ -129,18 +67,15 @@ xml.__path__ = [x for x in xml.__path__ if "_xmlplus" not in x]
 
 from xml.dom import minidom
 
-from zenmapGUI.higwidgets.higboxes import HIGHBox
 from zenmapGUI.higwidgets.higlabels import HIGEntryLabel
 from zenmapGUI.higwidgets.higbuttons import HIGButton
 
 from zenmapGUI.FileChoosers import AllFilesFileChooserDialog
 from zenmapGUI.ProfileHelp import ProfileHelp
 
-from zenmapCore.Paths import Path
 from zenmapCore.NmapOptions import NmapOptions, split_quoted, join_quoted
-import zenmapCore.I18N
-from zenmapCore.UmitLogging import log
-from zenmapGUI.ScriptInterface import *
+import zenmapCore.I18N  # lgtm[py/unused-import]
+from zenmapGUI.ScriptInterface import ScriptInterface
 
 
 def get_option_check_auxiliary_widget(option, ops, check):
@@ -165,9 +100,9 @@ def get_option_check_auxiliary_widget(option, ops, check):
         assert False, "Unknown option %s" % option
 
 
-class OptionEntry(gtk.Entry):
+class OptionEntry(Gtk.Entry):
     def __init__(self, option, ops, check):
-        gtk.Entry.__init__(self)
+        Gtk.Entry.__init__(self)
         self.option = option
         self.ops = ops
         self.check = check
@@ -185,18 +120,18 @@ class OptionEntry(gtk.Entry):
 
     def check_toggled_cb(self, check):
         if check.get_active():
-            self.ops[self.option] = self.get_text().decode("UTF-8")
+            self.ops[self.option] = self.get_text()
         else:
             self.ops[self.option] = None
 
     def changed_cb(self, widget):
         self.check.set_active(True)
-        self.ops[self.option] = self.get_text().decode("UTF-8")
+        self.ops[self.option] = self.get_text()
 
 
-class OptionExtras(gtk.Entry):
+class OptionExtras(Gtk.Entry):
     def __init__(self, option, ops, check):
-        gtk.Entry.__init__(self)
+        Gtk.Entry.__init__(self)
         self.ops = ops
         self.check = check
         self.connect("changed", self.changed_cb)
@@ -213,18 +148,19 @@ class OptionExtras(gtk.Entry):
 
     def check_toggled_cb(self, check):
         if check.get_active():
-            self.ops.extras = [self.get_text().decode("UTF-8")]
+            self.ops.extras = [self.get_text()]
         else:
             self.ops.extras = []
 
     def changed_cb(self, widget):
         self.check.set_active(True)
-        self.ops.extras = [self.get_text().decode("UTF-8")]
+        self.ops.extras = [self.get_text()]
 
 
-class OptionLevel(gtk.SpinButton):
+class OptionLevel(Gtk.SpinButton):
     def __init__(self, option, ops, check):
-        gtk.SpinButton.__init__(self, gtk.Adjustment(0, 0, 10, 1), 0.0, 0)
+        adjustment = Gtk.Adjustment.new(0, 0, 10, 1, 0, 0)
+        Gtk.SpinButton.__init__(self, adjustment=adjustment, climb_rate=0.0, digits=0)
         self.option = option
         self.ops = ops
         self.check = check
@@ -252,22 +188,22 @@ class OptionLevel(gtk.SpinButton):
         self.ops[self.option] = int(self.get_adjustment().get_value())
 
 
-class OptionFile(gtk.HBox):
+class OptionFile(Gtk.Box):
     __gsignals__ = {
-        "changed": (gobject.SIGNAL_RUN_FIRST, gobject.TYPE_NONE, ())
+        "changed": (GObject.SignalFlags.RUN_FIRST, GObject.TYPE_NONE, ())
     }
 
     def __init__(self, option, ops, check):
-        gtk.HBox.__init__(self)
+        Gtk.Box.__init__(self, orientation=Gtk.Orientation.HORIZONTAL)
 
         self.option = option
         self.ops = ops
         self.check = check
 
-        self.entry = gtk.Entry()
-        self.pack_start(self.entry, True, True)
-        button = HIGButton(stock=gtk.STOCK_OPEN)
-        self.pack_start(button, False)
+        self.entry = Gtk.Entry()
+        self.pack_start(self.entry, True, True, 0)
+        button = HIGButton(stock=Gtk.STOCK_OPEN)
+        self.pack_start(button, False, True, 0)
 
         button.connect("clicked", self.clicked_cb)
 
@@ -286,36 +222,36 @@ class OptionFile(gtk.HBox):
 
     def check_toggled_cb(self, check):
         if check.get_active():
-            self.ops[self.option] = self.entry.get_text().decode("UTF-8")
+            self.ops[self.option] = self.entry.get_text()
         else:
             self.ops[self.option] = None
 
     def changed_cb(self, widget):
         self.check.set_active(True)
-        self.ops[self.option] = self.entry.get_text().decode("UTF-8")
+        self.ops[self.option] = self.entry.get_text()
 
     def clicked_cb(self, button):
         dialog = AllFilesFileChooserDialog(_("Choose file"))
-        if dialog.run() == gtk.RESPONSE_OK:
+        if dialog.run() == Gtk.ResponseType.OK:
             self.entry.set_text(dialog.get_filename())
         dialog.destroy()
 
 
-class TargetEntry(gtk.Entry):
+class TargetEntry(Gtk.Entry):
     def __init__(self, ops):
-        gtk.Entry.__init__(self)
+        Gtk.Entry.__init__(self)
         self.ops = ops
         self.connect("changed", self.changed_cb)
         self.update()
 
     def update(self):
-        self.set_text(u" ".join(self.ops.target_specs))
+        self.set_text(" ".join(self.ops.target_specs))
 
     def changed_cb(self, widget):
         self.ops.target_specs = self.get_targets()
 
     def get_targets(self):
-        return split_quoted(self.get_text().decode("UTF-8"))
+        return split_quoted(self.get_text())
 
 
 class OptionTab(object):
@@ -339,29 +275,29 @@ class OptionTab(object):
                 self.widgets_list.append(widget)
 
     def __parse_target(self, target_element):
-        label = _(target_element.getAttribute(u'label'))
+        label = _(target_element.getAttribute('label'))
         label_widget = HIGEntryLabel(label)
         target_widget = TargetEntry(self.ops)
         target_widget.connect("changed", self.update_target)
         return label_widget, target_widget
 
     def __parse_option_list(self, option_list_element):
-        children = option_list_element.getElementsByTagName(u'option')
+        children = option_list_element.getElementsByTagName('option')
 
         label_widget = HIGEntryLabel(
-                _(option_list_element.getAttribute(u'label')))
+                _(option_list_element.getAttribute('label')))
         option_list_widget = OptionList(self.ops)
 
         for child in children:
-            option = child.getAttribute(u'option')
-            argument = child.getAttribute(u'argument')
-            label = _(child.getAttribute(u'label'))
+            option = child.getAttribute('option')
+            argument = child.getAttribute('argument')
+            label = _(child.getAttribute('label'))
             option_list_widget.append(option, argument, label)
             self.profilehelp.add_label(option, label)
             self.profilehelp.add_shortdesc(
-                    option, _(child.getAttribute(u'short_desc')))
+                    option, _(child.getAttribute('short_desc')))
             self.profilehelp.add_example(
-                    option, child.getAttribute(u'example'))
+                    option, child.getAttribute('example'))
 
         option_list_widget.update()
 
@@ -370,11 +306,10 @@ class OptionTab(object):
         return label_widget, option_list_widget
 
     def __parse_option_check(self, option_check):
-        arg_type = option_check.getAttribute(u'arg_type')
-        option = option_check.getAttribute(u'option')
-        label = _(option_check.getAttribute(u'label'))
-        short_desc = _(option_check.getAttribute(u'short_desc'))
-        example = option_check.getAttribute(u'example')
+        option = option_check.getAttribute('option')
+        label = _(option_check.getAttribute('label'))
+        short_desc = _(option_check.getAttribute('short_desc'))
+        example = option_check.getAttribute('example')
 
         self.profilehelp.add_label(option, label)
         self.profilehelp.add_shortdesc(option, short_desc)
@@ -396,7 +331,7 @@ class OptionTab(object):
         return check, auxiliary_widget
 
     def fill_table(self, table, expand_fill=True):
-        yopt = (0, gtk.EXPAND | gtk.FILL)[expand_fill]
+        yopt = (0, Gtk.AttachOptions.EXPAND | Gtk.AttachOptions.FILL)[expand_fill]
         for y, widget in enumerate(self.widgets_list):
             if widget[1] is None:
                 table.attach(widget[0], 0, 2, y, y + 1, yoptions=yopt)
@@ -492,12 +427,12 @@ class OptionBuilder(object):
         dic = {}
         for group in self.groups:
             grp = self.xml.getElementsByTagName(group)[0]
-            dic[group] = grp.getAttribute(u'label')
+            dic[group] = grp.getAttribute('label')
         return dic
 
     def __parse_groups(self):
-        return [g_name.getAttribute(u'name') for g_name in
-                self.xml.getElementsByTagName(u'groups')[0].getElementsByTagName(u'group')]  # noqa
+        return [g_name.getAttribute('name') for g_name in
+                self.xml.getElementsByTagName('groups')[0].getElementsByTagName('group')]  # noqa
 
     def __parse_tabs(self):
         dic = {}
@@ -513,14 +448,14 @@ class OptionBuilder(object):
         return dic
 
 
-class OptionList(gtk.ComboBox):
+class OptionList(Gtk.ComboBox):
     def __init__(self, ops):
         self.ops = ops
 
-        self.list = gtk.ListStore(str, str, str)
-        gtk.ComboBox.__init__(self, self.list)
+        self.list = Gtk.ListStore.new([str, str, str])
+        Gtk.ComboBox.__init__(self, model=self.list)
 
-        cell = gtk.CellRendererText()
+        cell = Gtk.CellRendererText()
         self.pack_start(cell, True)
         self.add_attribute(cell, 'text', 2)
 
@@ -552,12 +487,12 @@ class OptionList(gtk.ComboBox):
         self.options.append(option)
 
 
-class OptionCheck(gtk.CheckButton):
+class OptionCheck(Gtk.CheckButton):
     def __init__(self, option, label):
         opt = label
         if option is not None and option != "":
             opt += " (%s)" % option
 
-        gtk.CheckButton.__init__(self, opt, use_underline=False)
+        Gtk.CheckButton.__init__(self, label=opt, use_underline=False)
 
         self.option = option
