@@ -1,132 +1,64 @@
-#!/usr/bin/env python
-# -*- coding: utf-8 -*-
+#!/usr/bin/env python3
 
 # ***********************IMPORTANT NMAP LICENSE TERMS************************
-# *                                                                         *
-# * The Nmap Security Scanner is (C) 1996-2017 Insecure.Com LLC ("The Nmap  *
-# * Project"). Nmap is also a registered trademark of the Nmap Project.     *
-# * This program is free software; you may redistribute and/or modify it    *
-# * under the terms of the GNU General Public License as published by the   *
-# * Free Software Foundation; Version 2 ("GPL"), BUT ONLY WITH ALL OF THE   *
-# * CLARIFICATIONS AND EXCEPTIONS DESCRIBED HEREIN.  This guarantees your   *
-# * right to use, modify, and redistribute this software under certain      *
-# * conditions.  If you wish to embed Nmap technology into proprietary      *
-# * software, we sell alternative licenses (contact sales@nmap.com).        *
-# * Dozens of software vendors already license Nmap technology such as      *
-# * host discovery, port scanning, OS detection, version detection, and     *
-# * the Nmap Scripting Engine.                                              *
-# *                                                                         *
-# * Note that the GPL places important restrictions on "derivative works",  *
-# * yet it does not provide a detailed definition of that term.  To avoid   *
-# * misunderstandings, we interpret that term as broadly as copyright law   *
-# * allows.  For example, we consider an application to constitute a        *
-# * derivative work for the purpose of this license if it does any of the   *
-# * following with any software or content covered by this license          *
-# * ("Covered Software"):                                                   *
-# *                                                                         *
-# * o Integrates source code from Covered Software.                         *
-# *                                                                         *
-# * o Reads or includes copyrighted data files, such as Nmap's nmap-os-db   *
-# * or nmap-service-probes.                                                 *
-# *                                                                         *
-# * o Is designed specifically to execute Covered Software and parse the    *
-# * results (as opposed to typical shell or execution-menu apps, which will *
-# * execute anything you tell them to).                                     *
-# *                                                                         *
-# * o Includes Covered Software in a proprietary executable installer.  The *
-# * installers produced by InstallShield are an example of this.  Including *
-# * Nmap with other software in compressed or archival form does not        *
-# * trigger this provision, provided appropriate open source decompression  *
-# * or de-archiving software is widely available for no charge.  For the    *
-# * purposes of this license, an installer is considered to include Covered *
-# * Software even if it actually retrieves a copy of Covered Software from  *
-# * another source during runtime (such as by downloading it from the       *
-# * Internet).                                                              *
-# *                                                                         *
-# * o Links (statically or dynamically) to a library which does any of the  *
-# * above.                                                                  *
-# *                                                                         *
-# * o Executes a helper program, module, or script to do any of the above.  *
-# *                                                                         *
-# * This list is not exclusive, but is meant to clarify our interpretation  *
-# * of derived works with some common examples.  Other people may interpret *
-# * the plain GPL differently, so we consider this a special exception to   *
-# * the GPL that we apply to Covered Software.  Works which meet any of     *
-# * these conditions must conform to all of the terms of this license,      *
-# * particularly including the GPL Section 3 requirements of providing      *
-# * source code and allowing free redistribution of the work as a whole.    *
-# *                                                                         *
-# * As another special exception to the GPL terms, the Nmap Project grants  *
-# * permission to link the code of this program with any version of the     *
-# * OpenSSL library which is distributed under a license identical to that  *
-# * listed in the included docs/licenses/OpenSSL.txt file, and distribute   *
-# * linked combinations including the two.                                  *
-# *                                                                         *
-# * The Nmap Project has permission to redistribute Npcap, a packet         *
-# * capturing driver and library for the Microsoft Windows platform.        *
-# * Npcap is a separate work with it's own license rather than this Nmap    *
-# * license.  Since the Npcap license does not permit redistribution        *
-# * without special permission, our Nmap Windows binary packages which      *
-# * contain Npcap may not be redistributed without special permission.      *
-# *                                                                         *
-# * Any redistribution of Covered Software, including any derived works,    *
-# * must obey and carry forward all of the terms of this license, including *
-# * obeying all GPL rules and restrictions.  For example, source code of    *
-# * the whole work must be provided and free redistribution must be         *
-# * allowed.  All GPL references to "this License", are to be treated as    *
-# * including the terms and conditions of this license text as well.        *
-# *                                                                         *
-# * Because this license imposes special exceptions to the GPL, Covered     *
-# * Work may not be combined (even as part of a larger work) with plain GPL *
-# * software.  The terms, conditions, and exceptions of this license must   *
-# * be included as well.  This license is incompatible with some other open *
-# * source licenses as well.  In some cases we can relicense portions of    *
-# * Nmap or grant special permissions to use it in other open source        *
-# * software.  Please contact fyodor@nmap.org with any such requests.       *
-# * Similarly, we don't incorporate incompatible open source software into  *
-# * Covered Software without special permission from the copyright holders. *
-# *                                                                         *
-# * If you have any questions about the licensing restrictions on using     *
-# * Nmap in other works, are happy to help.  As mentioned above, we also    *
-# * offer alternative license to integrate Nmap into proprietary            *
-# * applications and appliances.  These contracts have been sold to dozens  *
-# * of software vendors, and generally include a perpetual license as well  *
-# * as providing for priority support and updates.  They also fund the      *
-# * continued development of Nmap.  Please email sales@nmap.com for further *
-# * information.                                                            *
-# *                                                                         *
-# * If you have received a written license agreement or contract for        *
-# * Covered Software stating terms other than these, you may choose to use  *
-# * and redistribute Covered Software under those terms instead of these.   *
-# *                                                                         *
-# * Source is provided to this software because we believe users have a     *
-# * right to know exactly what a program is going to do before they run it. *
-# * This also allows you to audit the software for security holes.          *
-# *                                                                         *
-# * Source code also allows you to port Nmap to new platforms, fix bugs,    *
-# * and add new features.  You are highly encouraged to send your changes   *
-# * to the dev@nmap.org mailing list for possible incorporation into the    *
-# * main distribution.  By sending these changes to Fyodor or one of the    *
-# * Insecure.Org development mailing lists, or checking them into the Nmap  *
-# * source code repository, it is understood (unless you specify            *
-# * otherwise) that you are offering the Nmap Project the unlimited,        *
-# * non-exclusive right to reuse, modify, and relicense the code.  Nmap     *
-# * will always be available Open Source, but this is important because     *
-# * the inability to relicense code has caused devastating problems for     *
-# * other Free Software projects (such as KDE and NASM).  We also           *
-# * occasionally relicense the code to third parties as discussed above.    *
-# * If you wish to specify special license conditions of your               *
-# * contributions, just say so when you send them.                          *
-# *                                                                         *
-# * This program is distributed in the hope that it will be useful, but     *
-# * WITHOUT ANY WARRANTY; without even the implied warranty of              *
-# * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the Nmap      *
-# * license file for more details (it's in a COPYING file included with     *
-# * Nmap, and also available from https://svn.nmap.org/nmap/COPYING)        *
-# *                                                                         *
+# *
+# * The Nmap Security Scanner is (C) 1996-2023 Nmap Software LLC ("The Nmap
+# * Project"). Nmap is also a registered trademark of the Nmap Project.
+# *
+# * This program is distributed under the terms of the Nmap Public Source
+# * License (NPSL). The exact license text applying to a particular Nmap
+# * release or source code control revision is contained in the LICENSE
+# * file distributed with that version of Nmap or source code control
+# * revision. More Nmap copyright/legal information is available from
+# * https://nmap.org/book/man-legal.html, and further information on the
+# * NPSL license itself can be found at https://nmap.org/npsl/ . This
+# * header summarizes some key points from the Nmap license, but is no
+# * substitute for the actual license text.
+# *
+# * Nmap is generally free for end users to download and use themselves,
+# * including commercial use. It is available from https://nmap.org.
+# *
+# * The Nmap license generally prohibits companies from using and
+# * redistributing Nmap in commercial products, but we sell a special Nmap
+# * OEM Edition with a more permissive license and special features for
+# * this purpose. See https://nmap.org/oem/
+# *
+# * If you have received a written Nmap license agreement or contract
+# * stating terms other than these (such as an Nmap OEM license), you may
+# * choose to use and redistribute Nmap under those terms instead.
+# *
+# * The official Nmap Windows builds include the Npcap software
+# * (https://npcap.com) for packet capture and transmission. It is under
+# * separate license terms which forbid redistribution without special
+# * permission. So the official Nmap Windows builds may not be redistributed
+# * without special permission (such as an Nmap OEM license).
+# *
+# * Source is provided to this software because we believe users have a
+# * right to know exactly what a program is going to do before they run it.
+# * This also allows you to audit the software for security holes.
+# *
+# * Source code also allows you to port Nmap to new platforms, fix bugs, and add
+# * new features. You are highly encouraged to submit your changes as a Github PR
+# * or by email to the dev@nmap.org mailing list for possible incorporation into
+# * the main distribution. Unless you specify otherwise, it is understood that
+# * you are offering us very broad rights to use your submissions as described in
+# * the Nmap Public Source License Contributor Agreement. This is important
+# * because we fund the project by selling licenses with various terms, and also
+# * because the inability to relicense code has caused devastating problems for
+# * other Free Software projects (such as KDE and NASM).
+# *
+# * The free version of Nmap is distributed in the hope that it will be
+# * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
+# * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. Warranties,
+# * indemnification and commercial support are all available through the
+# * Npcap OEM program--see https://nmap.org/oem/
+# *
 # ***************************************************************************/
 
-import gtk
+import gi
+
+gi.require_version("Gtk", "3.0")
+from gi.repository import Gtk
 
 from zenmapGUI.higwidgets.higwindows import HIGWindow
 from zenmapGUI.higwidgets.higboxes import HIGVBox, HIGHBox, HIGSpacer, \
@@ -137,11 +69,11 @@ from zenmapGUI.higwidgets.higtextviewers import HIGTextView
 from zenmapGUI.higwidgets.higbuttons import HIGButton
 from zenmapGUI.higwidgets.higtables import HIGTable
 from zenmapGUI.higwidgets.higdialogs import HIGAlertDialog, HIGDialog
-from zenmapGUI.OptionBuilder import *
+from zenmapGUI.OptionBuilder import OptionBuilder
 from zenmapCore.Paths import Path
-from zenmapCore.UmitConf import Profile, CommandProfile
+from zenmapCore.UmitConf import CommandProfile
 from zenmapCore.UmitLogging import log
-import zenmapCore.I18N
+import zenmapCore.I18N  # lgtm[py/unused-import]
 from zenmapCore.NmapOptions import NmapOptions
 
 
@@ -151,7 +83,7 @@ class ProfileEditor(HIGWindow):
         HIGWindow.__init__(self)
         self.connect("delete_event", self.exit)
         self.set_title(_('Profile Editor'))
-        self.set_position(gtk.WIN_POS_CENTER)
+        self.set_position(Gtk.WindowPosition.CENTER)
 
         self.deletable = deletable
         self.profile_name = profile_name
@@ -198,7 +130,7 @@ class ProfileEditor(HIGWindow):
         self.update_command()
 
     def command_entry_changed_cb(self, widget):
-        command_string = self.command_entry.get_text().decode("UTF-8")
+        command_string = self.command_entry.get_text()
         self.ops.parse_string(command_string)
         self.inhibit_command_update = True
         self.option_builder.update()
@@ -235,20 +167,21 @@ class ProfileEditor(HIGWindow):
         self.lower_box = HIGHBox()
 
         #self.main_vbox = HIGVBox()
-        self.command_entry = gtk.Entry()
+        self.command_entry = Gtk.Entry()
         self.command_entry_changed_cb_id = self.command_entry.connect(
                 "changed", self.command_entry_changed_cb)
 
         self.scan_button = HIGButton(_("Scan"))
         self.scan_button.connect("clicked", self.run_scan)
 
-        self.notebook = gtk.Notebook()
+        self.notebook = Gtk.Notebook()
 
         # Profile info page
         self.profile_info_vbox = HIGVBox()
         self.profile_info_label = HIGSectionLabel(_('Profile Information'))
         self.profile_name_label = HIGEntryLabel(_('Profile name'))
-        self.profile_name_entry = gtk.Entry()
+        self.profile_name_label.set_line_wrap(False)
+        self.profile_name_entry = Gtk.Entry()
         self.profile_name_entry.connect(
                 'enter-notify-event', self.update_help_name)
         self.profile_description_label = HIGEntryLabel(_('Description'))
@@ -261,13 +194,13 @@ class ProfileEditor(HIGWindow):
         # Buttons
         self.buttons_hbox = HIGHBox()
 
-        self.cancel_button = HIGButton(stock=gtk.STOCK_CANCEL)
+        self.cancel_button = HIGButton(stock=Gtk.STOCK_CANCEL)
         self.cancel_button.connect('clicked', self.exit)
 
-        self.delete_button = HIGButton(stock=gtk.STOCK_DELETE)
+        self.delete_button = HIGButton(stock=Gtk.STOCK_DELETE)
         self.delete_button.connect('clicked', self.delete_profile)
 
-        self.save_button = HIGButton(_("Save Changes"), stock=gtk.STOCK_SAVE)
+        self.save_button = HIGButton(_("Save Changes"), stock=Gtk.STOCK_SAVE)
         self.save_button.connect('clicked', self.save_profile)
 
         ###
@@ -296,7 +229,7 @@ class ProfileEditor(HIGWindow):
         self.middle_box._pack_expand_fill(self.help_vbox)
 
         # Packing buttons to lower box
-        self.lower_box.pack_end(self.buttons_hbox)
+        self.lower_box.pack_end(self.buttons_hbox, True, True, 0)
 
         # Packing the three vertical boxes to the main box
         self.main_whole_box._pack_noexpand_nofill(self.upper_box)
@@ -306,7 +239,7 @@ class ProfileEditor(HIGWindow):
 
         # Packing profile information tab on notebook
         self.notebook.append_page(
-                self.profile_info_vbox, gtk.Label(_('Profile')))
+                self.profile_info_vbox, Gtk.Label.new(_('Profile')))
         self.profile_info_vbox.set_border_width(5)
         table = HIGTable()
         self.profile_info_vbox._pack_noexpand_nofill(self.profile_info_label)
@@ -360,7 +293,7 @@ class ProfileEditor(HIGWindow):
         else:
             hbox = tab.get_hmain_box()
             vbox.pack_start(hbox, True, True, 0)
-        self.notebook.append_page(vbox, gtk.Label(tab_name))
+        self.notebook.append_page(vbox, Gtk.Label.new(tab_name))
 
     def save_profile(self, widget):
         if self.overwrite:
@@ -382,7 +315,7 @@ class ProfileEditor(HIGWindow):
 
         buf = self.profile_description_text.get_buffer()
         description = buf.get_text(
-                buf.get_start_iter(), buf.get_end_iter())
+                buf.get_start_iter(), buf.get_end_iter(), include_hidden_chars=True)
 
         try:
             self.profile.add_profile(
@@ -415,11 +348,11 @@ class ProfileEditor(HIGWindow):
 
     def delete_profile(self, widget=None, extra=None):
         if self.deletable:
-            dialog = HIGDialog(buttons=(gtk.STOCK_OK, gtk.RESPONSE_OK,
-                                        gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL))
+            dialog = HIGDialog(buttons=(Gtk.STOCK_OK, Gtk.ResponseType.OK,
+                                        Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL))
             alert = HIGEntryLabel('<b>' + _("Deleting Profile") + '</b>')
             text = HIGEntryLabel(_(
-                'Your profile is going to be deleted! ClickOk to continue, '
+                'Your profile is going to be deleted! Click Ok to continue, '
                 'or Cancel to go back to Profile Editor.'))
             hbox = HIGHBox()
             hbox.set_border_width(5)
@@ -429,21 +362,21 @@ class ProfileEditor(HIGWindow):
             vbox.set_border_width(5)
             vbox.set_spacing(12)
 
-            image = gtk.Image()
+            image = Gtk.Image()
             image.set_from_stock(
-                    gtk.STOCK_DIALOG_WARNING, gtk.ICON_SIZE_DIALOG)
+                    Gtk.STOCK_DIALOG_WARNING, Gtk.IconSize.DIALOG)
 
-            vbox.pack_start(alert)
-            vbox.pack_start(text)
-            hbox.pack_start(image)
-            hbox.pack_start(vbox)
+            vbox.pack_start(alert, True, True, 0)
+            vbox.pack_start(text, True, True, 0)
+            hbox.pack_start(image, True, True, 0)
+            hbox.pack_start(vbox, True, True, 0)
 
-            dialog.vbox.pack_start(hbox)
+            dialog.vbox.pack_start(hbox, True, True, 0)
             dialog.vbox.show_all()
 
             response = dialog.run()
             dialog.destroy()
-            if response == gtk.RESPONSE_CANCEL:
+            if response == Gtk.ResponseType.CANCEL:
                 return True
             self.profile.remove_profile(self.profile_name)
 
@@ -451,7 +384,7 @@ class ProfileEditor(HIGWindow):
         self.destroy()
 
     def run_scan(self, widget=None):
-        command_string = self.command_entry.get_text().decode("UTF-8")
+        command_string = self.command_entry.get_text()
         self.scan_interface.command_toolbar.command = command_string
         self.scan_interface.start_scan_cb()
         self.exit()
@@ -467,4 +400,4 @@ class ProfileEditor(HIGWindow):
 if __name__ == '__main__':
     p = ProfileEditor()
     p.show_all()
-    gtk.main()
+    Gtk.main()
